@@ -1,5 +1,5 @@
 // build/run probes
-// Copyright (C) 2005-2018 Red Hat Inc.
+// Copyright (C) 2005-2019 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -120,6 +120,16 @@ make_any_make_cmd(systemtap_session& s, const string& dir, const string& target)
       "CONFIG_STACK_VALIDATION=",
     };
 
+  // PR10280: suppress symbol versioning to restrict to exact kernel version
+  //
+  // XXX (PR24720): this was reported to cause problems on a PPC machine,
+  // and I've spotted kernel commits (43e24e82f35) which vary the vermagic
+  // string for PPC. Still, a standard PPC configuration tested fine, so
+  // I'm cautiously re-enabling this.
+  if (s.guru_mode)
+    make_cmd.push_back("CONFIG_MODVERSIONS=");
+  // Note: can re-enable from command line with "-B CONFIG_MODVERSIONS=y".
+
   // Add architecture, except for old powerpc (RHBZ669082)
   if (s.architecture != "powerpc" ||
       (strverscmp (s.kernel_base_release.c_str(), "2.6.15") >= 0))
@@ -150,13 +160,22 @@ make_make_objs_cmd(systemtap_session& s, const string& dir)
   //
   // So if we're only interested in the stage 1 objects, we can
   // cheat and make only the $(module-dirs) part.
-  return make_any_make_cmd(s, dir, "_module_" + dir);
+  //
+  // ... at least, before kernel commit c99f3918cf.
+  if (strverscmp (s.kernel_base_release.c_str(), "5.4") < 0)  
+    return make_any_make_cmd(s, dir, "_module_" + dir);
+  else
+    return make_any_make_cmd(s, dir, "modules"); // just build the lot
 }
 
 static void
-output_autoconf(systemtap_session& s, ofstream& o, const char *autoconf_c,
-                const char *deftrue, const char *deffalse)
+output_autoconf(systemtap_session& s, ofstream& o,
+                vector<string>& autoconf_c_files,
+                const char *autoconf_c, const char *deftrue,
+                const char *deffalse)
 {
+  autoconf_c_files.push_back (autoconf_c);
+  o << endl << s.tmpdir << "/" << autoconf_c << ".h:" << endl;
   o << "\t";
   if (s.verbose < 4)
     o << "@";
@@ -175,12 +194,8 @@ output_autoconf(systemtap_session& s, ofstream& o, const char *autoconf_c,
 void output_exportconf(systemtap_session& s, ofstream& o, const char *symbol,
                      const char *deftrue)
 {
-  o << "\t";
-  if (s.verbose < 4)
-    o << "@";
   if (s.kernel_exports.find(symbol) != s.kernel_exports.end())
-    o << "echo \"#define " << deftrue << " 1\"";
-  o << ">> $@" << endl;
+    o << "#define " << deftrue << " 1" << endl;
 }
 
 
@@ -188,13 +203,9 @@ void output_dual_exportconf(systemtap_session& s, ofstream& o,
 			    const char *symbol1, const char *symbol2,
 			    const char *deftrue)
 {
-  o << "\t";
-  if (s.verbose < 4)
-    o << "@";
   if (s.kernel_exports.find(symbol1) != s.kernel_exports.end()
       && s.kernel_exports.find(symbol2) != s.kernel_exports.end())
-    o << "echo \"#define " << deftrue << " 1\"";
-  o << ">> $@" << endl;
+    o << "#define " << deftrue << " 1" << endl;
 }
 
 
@@ -202,13 +213,9 @@ void output_either_exportconf(systemtap_session& s, ofstream& o,
 			      const char *symbol1, const char *symbol2,
 			      const char *deftrue)
 {
-  o << "\t";
-  if (s.verbose < 4)
-    o << "@";
   if (s.kernel_exports.find(symbol1) != s.kernel_exports.end()
       || s.kernel_exports.find(symbol2) != s.kernel_exports.end())
-    o << "echo \"#define " << deftrue << " 1\"";
-  o << ">> $@" << endl;
+    o << "#define " << deftrue << " 1" << endl;
 }
 
 
@@ -219,9 +226,10 @@ compile_dyninst (systemtap_session& s)
 
   vector<string> cmd
     {
-      "gcc", "--std=gnu99", s.translated_source, "-o", module,
+      "gcc", "--std=gnu99", s.translated_source, s.symbols_source, "-o", module,
       "-fvisibility=hidden", "-O2", "-I" + s.runtime_path, "-D__DYNINST__",
       "-Wall", WERROR, "-Wno-unused", "-Wno-strict-aliasing",
+      "-Wno-pointer-to-int-cast", "-Wno-int-to-pointer-cast",
       "-pthread", "-lrt", "-fPIC", "-shared",
     };
 
@@ -233,10 +241,8 @@ compile_dyninst (systemtap_session& s)
   if (s.architecture == "i386")
     cmd.push_back("-march=i586");
 
-  // Need this for scripts where tautologies pass through to generated
-  // code.  XXX: but this is gcc-version-dependent
-  cmd.push_back("-Wno-tautological-compare");
-  
+  cmd.push_back("-Wno-pragmas");
+
   for (size_t i = 0; i < s.c_macros.size(); ++i)
     cmd.push_back("-D" + s.c_macros[i]);
 
@@ -270,6 +276,9 @@ compile_pass (systemtap_session& s)
   string makefile_nm = s.tmpdir + "/Makefile";
   ofstream o (makefile_nm.c_str());
 
+  string stap_export_nm = s.tmpdir + "/stapconf_export.h";
+  ofstream o2 (stap_export_nm.c_str());
+
   // Create makefile
 
   // Clever hacks copied from vmware modules
@@ -282,7 +291,8 @@ compile_pass (systemtap_session& s)
     redirecterrors = "";
 
   // Support O= (or KBUILD_OUTPUT) option
-  o << "_KBUILD_CFLAGS := $(call flags,KBUILD_CFLAGS)" << endl;
+  // but flags= filter was removed from kernel scripts/Kbuild.include mid-2019
+  o << "_KBUILD_CFLAGS := $(call flags,KBUILD_CFLAGS) $(KBUILD_CFLAGS)" << endl;
 
   o << "stap_check_gcc = $(shell " << superverbose
     << " if $(CC) $(1) -S -o /dev/null -xc /dev/null > /dev/null 2>&1; then "
@@ -321,157 +331,211 @@ compile_pass (systemtap_session& s)
   // since such headers are cleansed of _KERNEL_ pieces that we need
 
   o << "STAPCONF_HEADER := " << s.tmpdir << "/" << s.stapconf_name << endl;
+  o << ".DELETE_ON_ERROR: $(STAPCONF_HEADER)" << endl;
   o << "$(STAPCONF_HEADER):" << endl;
-  o << "\t@> $@" << endl;
-  output_autoconf(s, o, "autoconf-hrtimer-rel.c", "STAPCONF_HRTIMER_REL", NULL);
-  output_exportconf(s, o, "hrtimer_get_res", "STAPCONF_HRTIMER_GET_RES");
-  output_autoconf(s, o, "autoconf-generated-compile.c", "STAPCONF_GENERATED_COMPILE", NULL);
-  output_autoconf(s, o, "autoconf-hrtimer-getset-expires.c", "STAPCONF_HRTIMER_GETSET_EXPIRES", NULL);
-  output_autoconf(s, o, "autoconf-inode-private.c", "STAPCONF_INODE_PRIVATE", NULL);
-  output_autoconf(s, o, "autoconf-inode-rwsem.c", "STAPCONF_INODE_RWSEM", NULL);
-  output_autoconf(s, o, "autoconf-constant-tsc.c", "STAPCONF_CONSTANT_TSC", NULL);
-  output_autoconf(s, o, "autoconf-ktime-get-real.c", "STAPCONF_KTIME_GET_REAL", NULL);
-  output_autoconf(s, o, "autoconf-x86-uniregs.c", "STAPCONF_X86_UNIREGS", NULL);
-  output_autoconf(s, o, "autoconf-nameidata.c", "STAPCONF_NAMEIDATA_CLEANUP", NULL);
-  output_dual_exportconf(s, o, "unregister_kprobes", "unregister_kretprobes", "STAPCONF_UNREGISTER_KPROBES");
-  output_autoconf(s, o, "autoconf-kprobe-symbol-name.c", "STAPCONF_KPROBE_SYMBOL_NAME", NULL);
-  output_autoconf(s, o, "autoconf-real-parent.c", "STAPCONF_REAL_PARENT", NULL);
-  output_autoconf(s, o, "autoconf-uaccess.c", "STAPCONF_LINUX_UACCESS_H", NULL);
-  output_autoconf(s, o, "autoconf-oneachcpu-retry.c", "STAPCONF_ONEACHCPU_RETRY", NULL);
-  output_autoconf(s, o, "autoconf-dpath-path.c", "STAPCONF_DPATH_PATH", NULL);
-  output_exportconf(s, o, "synchronize_kernel", "STAPCONF_SYNCHRONIZE_KERNEL");
-  output_exportconf(s, o, "synchronize_rcu", "STAPCONF_SYNCHRONIZE_RCU");
-  output_exportconf(s, o, "synchronize_sched", "STAPCONF_SYNCHRONIZE_SCHED");
-  output_autoconf(s, o, "autoconf-task-uid.c", "STAPCONF_TASK_UID", NULL);
-  output_autoconf(s, o, "autoconf-from_kuid_munged.c", "STAPCONF_FROM_KUID_MUNGED", NULL);
-  output_exportconf(s, o, "get_mm_exe_file", "STAPCONF_GET_MM_EXE_FILE");
-  output_dual_exportconf(s, o, "alloc_vm_area", "free_vm_area", "STAPCONF_VM_AREA");
-  output_autoconf(s, o, "autoconf-procfs-owner.c", "STAPCONF_PROCFS_OWNER", NULL);
-  output_autoconf(s, o, "autoconf-alloc-percpu-align.c", "STAPCONF_ALLOC_PERCPU_ALIGN", NULL);
-  output_autoconf(s, o, "autoconf-x86-fs.c", "STAPCONF_X86_FS", NULL);
-  output_autoconf(s, o, "autoconf-x86-xfs.c", "STAPCONF_X86_XFS", NULL);
-  output_autoconf(s, o, "autoconf-x86-gs.c", "STAPCONF_X86_GS", NULL);
-  output_autoconf(s, o, "autoconf-grsecurity.c", "STAPCONF_GRSECURITY", NULL);
-  output_autoconf(s, o, "autoconf-trace-printk.c", "STAPCONF_TRACE_PRINTK", NULL);
-  output_autoconf(s, o, "autoconf-regset.c", "STAPCONF_REGSET", NULL);
-  output_autoconf(s, o, "autoconf-utrace-regset.c", "STAPCONF_UTRACE_REGSET", NULL);
-  output_autoconf(s, o, "autoconf-uprobe-get-pc.c", "STAPCONF_UPROBE_GET_PC", NULL);
-  output_autoconf(s, o, "autoconf-hlist-4args.c", "STAPCONF_HLIST_4ARGS", NULL);
-  output_exportconf(s, o, "tsc_khz", "STAPCONF_TSC_KHZ");
-  output_exportconf(s, o, "cpu_khz", "STAPCONF_CPU_KHZ");
-  output_exportconf(s, o, "__module_text_address", "STAPCONF_MODULE_TEXT_ADDRESS");
-  output_exportconf(s, o, "add_timer_on", "STAPCONF_ADD_TIMER_ON");
+  o << "\t";
+  if (s.verbose < 4)
+    o << "@";
+  o << "$(MAKE) -f \"$(firstword $(MAKEFILE_LIST))\" gen-stapconf" << endl;
 
-  output_dual_exportconf(s, o, "probe_kernel_read", "probe_kernel_write", "STAPCONF_PROBE_KERNEL");
-  output_autoconf(s, o, "autoconf-hw_breakpoint_context.c",
+  vector<string> cs;  // to hold autoconf C file names
+
+  output_autoconf(s, o, cs, "autoconf-hrtimer-rel.c", "STAPCONF_HRTIMER_REL", NULL);
+  output_exportconf(s, o2, "hrtimer_get_res", "STAPCONF_HRTIMER_GET_RES");
+  output_autoconf(s, o, cs, "autoconf-generated-compile.c", "STAPCONF_GENERATED_COMPILE", NULL);
+  output_autoconf(s, o, cs, "autoconf-hrtimer-getset-expires.c", "STAPCONF_HRTIMER_GETSET_EXPIRES", NULL);
+  output_autoconf(s, o, cs, "autoconf-inode-private.c", "STAPCONF_INODE_PRIVATE", NULL);
+  output_autoconf(s, o, cs, "autoconf-inode-rwsem.c", "STAPCONF_INODE_RWSEM", NULL);
+  output_autoconf(s, o, cs, "autoconf-constant-tsc.c", "STAPCONF_CONSTANT_TSC", NULL);
+  output_autoconf(s, o, cs, "autoconf-ktime-get-real.c", "STAPCONF_KTIME_GET_REAL", NULL);
+  output_exportconf(s, o2, "ktime_get_real_fast_ns", "STAPCONF_KTIME_GET_REAL_FAST_NS");
+  output_autoconf(s, o, cs, "autoconf-x86-uniregs.c", "STAPCONF_X86_UNIREGS", NULL);
+  output_autoconf(s, o, cs, "autoconf-nameidata.c", "STAPCONF_NAMEIDATA_CLEANUP", NULL);
+  output_dual_exportconf(s, o2, "unregister_kprobes", "unregister_kretprobes", "STAPCONF_UNREGISTER_KPROBES");
+  output_autoconf(s, o, cs, "autoconf-kprobe-symbol-name.c", "STAPCONF_KPROBE_SYMBOL_NAME", NULL);
+  output_autoconf(s, o, cs, "autoconf-real-parent.c", "STAPCONF_REAL_PARENT", NULL);
+  output_autoconf(s, o, cs, "autoconf-uaccess.c", "STAPCONF_LINUX_UACCESS_H", NULL);
+  output_autoconf(s, o, cs, "autoconf-oneachcpu-retry.c", "STAPCONF_ONEACHCPU_RETRY", NULL);
+  output_autoconf(s, o, cs, "autoconf-dpath-path.c", "STAPCONF_DPATH_PATH", NULL);
+  output_exportconf(s, o2, "synchronize_kernel", "STAPCONF_SYNCHRONIZE_KERNEL");
+  output_exportconf(s, o2, "synchronize_rcu", "STAPCONF_SYNCHRONIZE_RCU");
+  output_exportconf(s, o2, "synchronize_sched", "STAPCONF_SYNCHRONIZE_SCHED");
+  output_autoconf(s, o, cs, "autoconf-task-uid.c", "STAPCONF_TASK_UID", NULL);
+  output_autoconf(s, o, cs, "autoconf-from_kuid_munged.c", "STAPCONF_FROM_KUID_MUNGED", NULL);
+  output_exportconf(s, o2, "get_mm_exe_file", "STAPCONF_GET_MM_EXE_FILE");
+  output_dual_exportconf(s, o2, "alloc_vm_area", "free_vm_area", "STAPCONF_VM_AREA");
+  output_autoconf(s, o, cs, "autoconf-procfs-owner.c", "STAPCONF_PROCFS_OWNER", NULL);
+  output_autoconf(s, o, cs, "autoconf-alloc-percpu-align.c", "STAPCONF_ALLOC_PERCPU_ALIGN", NULL);
+  output_autoconf(s, o, cs, "autoconf-set_fs.c", "STAPCONF_SET_FS", NULL);
+  output_autoconf(s, o, cs, "autoconf-x86-fs.c", "STAPCONF_X86_FS", NULL);
+  output_autoconf(s, o, cs, "autoconf-x86-xfs.c", "STAPCONF_X86_XFS", NULL);
+  output_autoconf(s, o, cs, "autoconf-x86-gs.c", "STAPCONF_X86_GS", NULL);
+  output_autoconf(s, o, cs, "autoconf-grsecurity.c", "STAPCONF_GRSECURITY", NULL);
+  output_autoconf(s, o, cs, "autoconf-trace-printk.c", "STAPCONF_TRACE_PRINTK", NULL);
+  output_autoconf(s, o, cs, "autoconf-regset.c", "STAPCONF_REGSET", NULL);
+  output_autoconf(s, o, cs, "autoconf-utrace-regset.c", "STAPCONF_UTRACE_REGSET", NULL);
+  output_autoconf(s, o, cs, "autoconf-uprobe-get-pc.c", "STAPCONF_UPROBE_GET_PC", NULL);
+  output_autoconf(s, o, cs, "autoconf-hlist-4args.c", "STAPCONF_HLIST_4ARGS", NULL);
+  output_exportconf(s, o2, "tsc_khz", "STAPCONF_TSC_KHZ");
+  output_exportconf(s, o2, "cpu_khz", "STAPCONF_CPU_KHZ");
+  output_exportconf(s, o2, "__module_text_address", "STAPCONF_MODULE_TEXT_ADDRESS");
+  output_exportconf(s, o2, "add_timer_on", "STAPCONF_ADD_TIMER_ON");
+
+  output_dual_exportconf(s, o2, "probe_kernel_read", "probe_kernel_write", "STAPCONF_PROBE_KERNEL");
+  output_autoconf(s, o, cs, "autoconf-hw_breakpoint_context.c",
 		  "STAPCONF_HW_BREAKPOINT_CONTEXT", NULL);
-  output_autoconf(s, o, "autoconf-save-stack-trace.c",
+  output_exportconf(s, o2, "save_stack_trace_regs", "STAPCONF_SAVE_STACK_TRACE_REGS_EXPORTED");
+  output_autoconf(s, o, cs, "autoconf-save-stack-trace.c",
                   "STAPCONF_KERNEL_STACKTRACE", NULL);
-  output_autoconf(s, o, "autoconf-save-stack-trace-no-bp.c",
+  output_autoconf(s, o, cs, "autoconf-save-stack-trace-no-bp.c",
                   "STAPCONF_KERNEL_STACKTRACE_NO_BP", NULL);
-  output_autoconf(s, o, "autoconf-unwind-stack-trace.c",
+  output_autoconf(s, o, cs, "autoconf-unwind-stack-trace.c",
                   "STAPCONF_KERNEL_UNWIND_STACK", NULL);
-  output_autoconf(s, o, "autoconf-asm-syscall.c",
+  output_autoconf(s, o, cs, "autoconf-asm-syscall.c",
 		  "STAPCONF_ASM_SYSCALL_H", NULL);
-  output_autoconf(s, o, "autoconf-ring_buffer-flags.c", "STAPCONF_RING_BUFFER_FLAGS", NULL);
-  output_autoconf(s, o, "autoconf-ring_buffer_lost_events.c", "STAPCONF_RING_BUFFER_LOST_EVENTS", NULL);
-  output_autoconf(s, o, "autoconf-ring_buffer_read_prepare.c", "STAPCONF_RING_BUFFER_READ_PREPARE", NULL);
-  output_autoconf(s, o, "autoconf-kallsyms-on-each-symbol.c", "STAPCONF_KALLSYMS_ON_EACH_SYMBOL", NULL);
-  output_autoconf(s, o, "autoconf-walk-stack.c", "STAPCONF_WALK_STACK", NULL);
-  output_autoconf(s, o, "autoconf-stacktrace_ops-warning.c",
+  output_autoconf(s, o, cs, "autoconf-syscall_get_args_3args.c",
+		  "STAPCONF_SYSCALL_GET_ARGS_3ARGS", NULL);
+  output_autoconf(s, o, cs, "autoconf-ring_buffer-flags.c", "STAPCONF_RING_BUFFER_FLAGS", NULL);
+  output_autoconf(s, o, cs, "autoconf-ring_buffer_lost_events.c", "STAPCONF_RING_BUFFER_LOST_EVENTS", NULL);
+  output_autoconf(s, o, cs, "autoconf-ring_buffer_read_prepare.c", "STAPCONF_RING_BUFFER_READ_PREPARE", NULL);
+  output_exportconf(s, o2, "kallsyms_on_each_symbol", "STAPCONF_KALLSYMS_ON_EACH_SYMBOL_EXPORTED");
+
+  output_autoconf(s, o, cs, "autoconf-walk-stack.c", "STAPCONF_WALK_STACK", NULL);
+  output_autoconf(s, o, cs, "autoconf-stacktrace_ops-warning.c",
                   "STAPCONF_STACKTRACE_OPS_WARNING", NULL);
-  output_autoconf(s, o, "autoconf-stacktrace_ops-int-address.c",
+  output_autoconf(s, o, cs, "autoconf-stacktrace_ops-int-address.c",
                   "STAPCONF_STACKTRACE_OPS_INT_ADDRESS", NULL);
-  output_autoconf(s, o, "autoconf-mm-context-vdso.c", "STAPCONF_MM_CONTEXT_VDSO", NULL);
-  output_autoconf(s, o, "autoconf-mm-context-vdso-base.c", "STAPCONF_MM_CONTEXT_VDSO_BASE", NULL);
-  output_autoconf(s, o, "autoconf-blk-types.c", "STAPCONF_BLK_TYPES", NULL);
-  output_autoconf(s, o, "autoconf-perf-structpid.c", "STAPCONF_PERF_STRUCTPID", NULL);
-  output_autoconf(s, o, "perf_event_counter_context.c",
+  output_autoconf(s, o, cs, "autoconf-mm-context-vdso.c", "STAPCONF_MM_CONTEXT_VDSO", NULL);
+  output_autoconf(s, o, cs, "autoconf-mm-context-vdso-base.c", "STAPCONF_MM_CONTEXT_VDSO_BASE", NULL);
+  output_autoconf(s, o, cs, "autoconf-blk-types.c", "STAPCONF_BLK_TYPES", NULL);
+  output_autoconf(s, o, cs, "autoconf-perf-structpid.c", "STAPCONF_PERF_STRUCTPID", NULL);
+  output_autoconf(s, o, cs, "perf_event_counter_context.c",
 		  "STAPCONF_PERF_COUNTER_CONTEXT", NULL);
-  output_autoconf(s, o, "perf_probe_handler_nmi.c",
+  output_autoconf(s, o, cs, "perf_probe_handler_nmi.c",
 		  "STAPCONF_PERF_HANDLER_NMI", NULL);
-  output_exportconf(s, o, "path_lookup", "STAPCONF_PATH_LOOKUP");
-  output_exportconf(s, o, "kern_path_parent", "STAPCONF_KERN_PATH_PARENT");
-  output_exportconf(s, o, "vfs_path_lookup", "STAPCONF_VFS_PATH_LOOKUP");
-  output_exportconf(s, o, "kern_path", "STAPCONF_KERN_PATH");
-  output_exportconf(s, o, "proc_create_data", "STAPCONF_PROC_CREATE_DATA");
-  output_exportconf(s, o, "PDE_DATA", "STAPCONF_PDE_DATA");
-  output_autoconf(s, o, "autoconf-module-sect-attrs.c", "STAPCONF_MODULE_SECT_ATTRS", NULL);
-
-  output_autoconf(s, o, "autoconf-utrace-via-tracepoints.c", "STAPCONF_UTRACE_VIA_TRACEPOINTS", NULL);
-  output_autoconf(s, o, "autoconf-task_work-struct.c", "STAPCONF_TASK_WORK_STRUCT", NULL);
-  output_autoconf(s, o, "autoconf-vm-area-pte.c", "STAPCONF_VM_AREA_PTE", NULL);
-  output_autoconf(s, o, "autoconf-relay-umode_t.c", "STAPCONF_RELAY_UMODE_T", NULL);
-  output_autoconf(s, o, "autoconf-relay_buf-per_cpu_ptr.c", "STAPCONF_RELAY_BUF_PER_CPU_PTR", NULL);
-  output_autoconf(s, o, "autoconf-fs_supers-hlist.c", "STAPCONF_FS_SUPERS_HLIST", NULL);
-  output_autoconf(s, o, "autoconf-compat_sigaction.c", "STAPCONF_COMPAT_SIGACTION", NULL);
-  output_autoconf(s, o, "autoconf-netfilter.c", "STAPCONF_NETFILTER_V313", NULL);
-  output_autoconf(s, o, "autoconf-netfilter-313b.c", "STAPCONF_NETFILTER_V313B", NULL);
-  output_autoconf(s, o, "autoconf-netfilter-4_1.c", "STAPCONF_NETFILTER_V41", NULL);
-  output_autoconf(s, o, "autoconf-netfilter-4_4.c", "STAPCONF_NETFILTER_V44", NULL);
-  output_autoconf(s, o, "autoconf-smpcall-5args.c", "STAPCONF_SMPCALL_5ARGS", NULL);
-  output_autoconf(s, o, "autoconf-smpcall-4args.c", "STAPCONF_SMPCALL_4ARGS", NULL);
-
+  output_exportconf(s, o2, "path_lookup", "STAPCONF_PATH_LOOKUP");
+  output_exportconf(s, o2, "kern_path_parent", "STAPCONF_KERN_PATH_PARENT");
+  output_exportconf(s, o2, "vfs_path_lookup", "STAPCONF_VFS_PATH_LOOKUP");
+  output_exportconf(s, o2, "kern_path", "STAPCONF_KERN_PATH");
+  output_exportconf(s, o2, "proc_create_data", "STAPCONF_PROC_CREATE_DATA");
+  output_autoconf(s, o, cs, "autoconf-proc_ops.c", "STAPCONF_PROC_OPS", NULL);
+  output_exportconf(s, o2, "PDE_DATA", "STAPCONF_PDE_DATA");
+  output_autoconf(s, o, cs, "autoconf-module-sect-attrs.c", "STAPCONF_MODULE_SECT_ATTRS", NULL);
+  output_exportconf(s, o2, "kernel_read_file_from_path", "STAPCONF_KERNEL_READ_FILE_FROM_PATH");
+  output_autoconf(s, o, cs, "autoconf-kernel_read_file_from_path-offset.c", "STAPCONF_KERNEL_READ_FILE_FROM_PATH_OFFSET", NULL);
+  output_autoconf(s, o, cs, "autoconf-utrace-via-tracepoints.c", "STAPCONF_UTRACE_VIA_TRACEPOINTS", NULL);
+  output_autoconf(s, o, cs, "autoconf-task_work-struct.c", "STAPCONF_TASK_WORK_STRUCT", NULL);
+  output_autoconf(s, o, cs, "autoconf-vm-area-pte.c", "STAPCONF_VM_AREA_PTE", NULL);
+  output_autoconf(s, o, cs, "autoconf-relay-umode_t.c", "STAPCONF_RELAY_UMODE_T", NULL);
+  output_autoconf(s, o, cs, "autoconf-relay_buf-per_cpu_ptr.c", "STAPCONF_RELAY_BUF_PER_CPU_PTR", NULL);
+  output_autoconf(s, o, cs, "autoconf-fs_supers-hlist.c", "STAPCONF_FS_SUPERS_HLIST", NULL);
+  output_autoconf(s, o, cs, "autoconf-compat_sigaction.c", "STAPCONF_COMPAT_SIGACTION", NULL);
+  output_autoconf(s, o, cs, "autoconf-netfilter.c", "STAPCONF_NETFILTER_V313", NULL);
+  output_autoconf(s, o, cs, "autoconf-netfilter-313b.c", "STAPCONF_NETFILTER_V313B", NULL);
+  output_autoconf(s, o, cs, "autoconf-netfilter-4_1.c", "STAPCONF_NETFILTER_V41", NULL);
+  output_autoconf(s, o, cs, "autoconf-netfilter-4_4.c", "STAPCONF_NETFILTER_V44", NULL);
+  output_autoconf(s, o, cs, "autoconf-smpcall-5args.c", "STAPCONF_SMPCALL_5ARGS", NULL);
+  output_autoconf(s, o, cs, "autoconf-smpcall-4args.c", "STAPCONF_SMPCALL_4ARGS", NULL);
+  output_autoconf(s, o, cs, "autoconf-access_ok_2args.c", "STAPCONF_ACCESS_OK_2ARGS", NULL);
+  output_autoconf(s, o, cs, "autoconf-uapi-mount.c", "STAPCONF_UAPI_LINUX_MOUNT_H", NULL);
+  output_autoconf(s, o, cs, "autoconf-time32.c", "STAPCONF_TIME32_H", NULL);
+  output_autoconf(s, o, cs, "autoconf-time32-old.c", "STAPCONF_TIME32_OLD_H", NULL);
+  output_autoconf(s, o, cs, "autoconf-compat-utimbuf.c", "STAPCONF_COMPAT_UTIMBUF", NULL);
+  
   // used by tapset/timestamp_monotonic.stp
-  output_autoconf(s, o, "autoconf-cpu-clock.c", "STAPCONF_CPU_CLOCK", NULL);
-  output_autoconf(s, o, "autoconf-local-clock.c", "STAPCONF_LOCAL_CLOCK", NULL);
+  output_autoconf(s, o, cs, "autoconf-cpu-clock.c", "STAPCONF_CPU_CLOCK", NULL);
+  output_autoconf(s, o, cs, "autoconf-local-clock.c", "STAPCONF_LOCAL_CLOCK", NULL);
+  
+  // used by tapset/linux/proc_mem.stp
+  output_autoconf(s, o, cs, "autoconf-mm-shmempages.c", "STAPCONF_MM_SHMEMPAGES", NULL);
 
   // used by runtime/uprobe-inode.c
-  output_either_exportconf(s, o, "uprobe_register", "register_uprobe",
+  output_either_exportconf(s, o2, "uprobe_register", "register_uprobe",
 			   "STAPCONF_UPROBE_REGISTER_EXPORTED");
-  output_either_exportconf(s, o, "uprobe_unregister", "unregister_uprobe",
+  output_either_exportconf(s, o2, "uprobe_unregister", "unregister_uprobe",
 			   "STAPCONF_UPROBE_UNREGISTER_EXPORTED");
-  output_autoconf(s, o, "autoconf-old-inode-uprobes.c", "STAPCONF_OLD_INODE_UPROBES", NULL);
-  output_autoconf(s, o, "autoconf-inode-uretprobes.c", "STAPCONF_INODE_URETPROBES", NULL);
+  output_autoconf(s, o, cs, "autoconf-old-inode-uprobes.c", "STAPCONF_OLD_INODE_UPROBES", NULL);
+  output_autoconf(s, o, cs, "autoconf-inode-uretprobes.c", "STAPCONF_INODE_URETPROBES", NULL);
 
   // used by tapsets.cxx inode uprobe generated code
-  output_exportconf(s, o, "uprobe_get_swbp_addr", "STAPCONF_UPROBE_GET_SWBP_ADDR_EXPORTED");
+  output_exportconf(s, o2, "uprobe_get_swbp_addr", "STAPCONF_UPROBE_GET_SWBP_ADDR_EXPORTED");
 
   // used by runtime/loc2c-runtime.h
-  output_exportconf(s, o, "task_user_regset_view", "STAPCONF_TASK_USER_REGSET_VIEW_EXPORTED");
+  output_exportconf(s, o2, "task_user_regset_view", "STAPCONF_TASK_USER_REGSET_VIEW_EXPORTED");
 
   // used by runtime/stp_utrace.c
-  output_exportconf(s, o, "task_work_add", "STAPCONF_TASK_WORK_ADD_EXPORTED");
-  output_exportconf(s, o, "wake_up_state", "STAPCONF_WAKE_UP_STATE_EXPORTED");
-  output_exportconf(s, o, "try_to_wake_up", "STAPCONF_TRY_TO_WAKE_UP_EXPORTED");
-  output_exportconf(s, o, "signal_wake_up_state", "STAPCONF_SIGNAL_WAKE_UP_STATE_EXPORTED");
-  output_exportconf(s, o, "signal_wake_up", "STAPCONF_SIGNAL_WAKE_UP_EXPORTED");
-  output_exportconf(s, o, "__lock_task_sighand", "STAPCONF___LOCK_TASK_SIGHAND_EXPORTED");
+  output_exportconf(s, o2, "task_work_add", "STAPCONF_TASK_WORK_ADD_EXPORTED");
+  output_exportconf(s, o2, "task_work_cancel", "STAPCONF_TASK_WORK_CANCEL_EXPORTED");
+  output_exportconf(s, o2, "wake_up_state", "STAPCONF_WAKE_UP_STATE_EXPORTED");
+  output_exportconf(s, o2, "try_to_wake_up", "STAPCONF_TRY_TO_WAKE_UP_EXPORTED");
+  output_exportconf(s, o2, "signal_wake_up_state", "STAPCONF_SIGNAL_WAKE_UP_STATE_EXPORTED");
+  output_exportconf(s, o2, "signal_wake_up", "STAPCONF_SIGNAL_WAKE_UP_EXPORTED");
+  output_exportconf(s, o2, "__lock_task_sighand", "STAPCONF___LOCK_TASK_SIGHAND_EXPORTED");
 
-  output_autoconf(s, o, "autoconf-pagefault_disable.c", "STAPCONF_PAGEFAULT_DISABLE", NULL);
-  output_exportconf(s, o, "kallsyms_lookup_name", "STAPCONF_KALLSYMS");
-  output_autoconf(s, o, "autoconf-uidgid.c", "STAPCONF_LINUX_UIDGID_H", NULL);
-  output_exportconf(s, o, "sigset_from_compat", "STAPCONF_SIGSET_FROM_COMPAT_EXPORTED");
-  output_exportconf(s, o, "vzalloc", "STAPCONF_VZALLOC");
-  output_exportconf(s, o, "vzalloc_node", "STAPCONF_VZALLOC_NODE");
-  output_exportconf(s, o, "vmalloc_node", "STAPCONF_VMALLOC_NODE");
+  output_autoconf(s, o, cs, "autoconf-pagefault_disable.c", "STAPCONF_PAGEFAULT_DISABLE", NULL);
+  output_exportconf(s, o2, "kallsyms_lookup_name", "STAPCONF_KALLSYMS_LOOKUP_NAME_EXPORTED");
+  output_autoconf(s, o, cs, "autoconf-uidgid.c", "STAPCONF_LINUX_UIDGID_H", NULL);
+  output_exportconf(s, o2, "sigset_from_compat", "STAPCONF_SIGSET_FROM_COMPAT_EXPORTED");
+  output_exportconf(s, o2, "vzalloc", "STAPCONF_VZALLOC");
+  output_exportconf(s, o2, "vzalloc_node", "STAPCONF_VZALLOC_NODE");
+  output_exportconf(s, o2, "vmalloc_node", "STAPCONF_VMALLOC_NODE");
 
   // RHBZ1233912 - s390 temporary workaround for non-atomic udelay()
-  output_exportconf(s, o, "udelay_simple", "STAPCONF_UDELAY_SIMPLE_EXPORTED");
-  output_autoconf(s, o, "autoconf-udelay_simple.c", "STAPCONF_UDELAY_SIMPLE",
+  output_exportconf(s, o2, "udelay_simple", "STAPCONF_UDELAY_SIMPLE_EXPORTED");
+  output_autoconf(s, o, cs, "autoconf-udelay_simple.c", "STAPCONF_UDELAY_SIMPLE",
 		  NULL);
 
-  output_autoconf(s, o, "autoconf-tracepoint-strings.c", "STAPCONF_TRACEPOINT_STRINGS", NULL);
-  output_autoconf(s, o, "autoconf-timerfd.c", "STAPCONF_TIMERFD_H", NULL);
+  // RHBZ1788662 - need rcu_is_watching()
+  output_autoconf(s, o, cs, "autoconf-rcu_is_watching.c", "STAPCONF_RCU_IS_WATCHING", NULL);
 
-  output_autoconf(s, o, "autoconf-module_layout.c",
+  output_autoconf(s, o, cs, "autoconf-tracepoint-has-data.c", "STAPCONF_TRACEPOINT_HAS_DATA", NULL);
+  output_autoconf(s, o, cs, "autoconf-tracepoint-strings.c", "STAPCONF_TRACEPOINT_STRINGS", NULL);
+  output_autoconf(s, o, cs, "autoconf-timerfd.c", "STAPCONF_TIMERFD_H", NULL);
+
+  output_autoconf(s, o, cs, "autoconf-module_layout.c",
 		  "STAPCONF_MODULE_LAYOUT", NULL);
-  output_autoconf(s, o, "autoconf-mod_kallsyms.c",
+  output_autoconf(s, o, cs, "autoconf-mod_kallsyms.c",
 		  "STAPCONF_MOD_KALLSYMS", NULL);
-  output_exportconf(s, o, "get_user_pages_remote", "STAPCONF_GET_USER_PAGES_REMOTE");
-  output_autoconf(s, o, "autoconf-get_user_pages_remote-flags.c",
+  output_exportconf(s, o2, "get_user_pages_remote", "STAPCONF_GET_USER_PAGES_REMOTE");
+  output_autoconf(s, o, cs, "autoconf-get_user_pages_remote-flags.c",
 		  "STAPCONF_GET_USER_PAGES_REMOTE_FLAGS", NULL);
-  output_autoconf(s, o, "autoconf-get_user_pages_remote-flags_locked.c",
+  output_autoconf(s, o, cs, "autoconf-get_user_pages_remote-flags_locked.c",
 		  "STAPCONF_GET_USER_PAGES_REMOTE_FLAGS_LOCKED", NULL);
-  output_autoconf(s, o, "autoconf-bio-bi_opf.c", "STAPCONF_BIO_BI_OPF", NULL);
-  output_autoconf(s, o, "autoconf-linux-sched_headers.c",
+  output_autoconf(s, o, cs, "autoconf-get_user_pages_remote-notask_struct.c",
+		  "STAPCONF_GET_USER_PAGES_REMOTE_NOTASK_STRUCT", NULL);
+  output_autoconf(s, o, cs, "autoconf-get_user_pages-flags.c",
+		  "STAPCONF_GET_USER_PAGES_FLAGS", NULL);
+  output_autoconf(s, o, cs, "autoconf-get_user_pages-notask_struct.c",
+		  "STAPCONF_GET_USER_PAGES_NOTASK_STRUCT", NULL);
+  output_autoconf(s, o, cs, "autoconf-bio-bi_opf.c", "STAPCONF_BIO_BI_OPF", NULL);
+  output_autoconf(s, o, cs, "autoconf-linux-sched_headers.c",
 		  "STAPCONF_LINUX_SCHED_HEADERS", NULL);
-
+  output_autoconf(s, o, cs, "autoconf-stack-trace-save-regs.c",
+		  "STAPCONF_STACK_TRACE_SAVE_REGS", NULL);
+  output_autoconf(s, o, cs, "autoconf-mmap_lock.c",
+		  "STAPCONF_MMAP_LOCK", NULL);
+  output_autoconf(s, o, cs, "autoconf-atomic_fetch_add_unless.c",
+		  "STAPCONF_ATOMIC_FETCH_ADD_UNLESS", NULL);
+  output_autoconf(s, o, cs, "autoconf-lockdown-debugfs.c", "STAPCONF_LOCKDOWN_DEBUGFS", NULL);
+  
   // used by runtime/linux/netfilter.c
-  output_exportconf(s, o, "nf_register_hook", "STAPCONF_NF_REGISTER_HOOK");
+  output_exportconf(s, o2, "nf_register_hook", "STAPCONF_NF_REGISTER_HOOK");
 
   // used by tapset/linux/ioblock.stp
-  output_exportconf(s, o, "disk_get_part", "STAPCONF_DISK_GET_PART");
+  output_exportconf(s, o2, "disk_get_part", "STAPCONF_DISK_GET_PART");
+
+  o2.close ();
+
+  o << ".PHONY: gen-stapconf" << endl;
+  o << "gen-stapconf: " << stap_export_nm;
+  for (unsigned i=0; i<cs.size(); i++)
+    o << " " << s.tmpdir << "/" << cs[i] << ".h";
+  o << endl;
+
+  o << "\t";
+  if (s.verbose < 4)
+    o << "@";
+  o << "cat $^ > $(STAPCONF_HEADER)" << endl;
 
   o << module_cflags << " += -include $(STAPCONF_HEADER)" << endl;
 
@@ -511,12 +575,19 @@ compile_pass (systemtap_session& s)
   o << "EXTRA_CFLAGS += $(call cc-option,-fno-ipa-icf)" << endl;
 
   // Assumes linux 2.6 kbuild
-  o << "EXTRA_CFLAGS += -Wno-unused $(call cc-option,-Wno-tautological-compare) " << WERROR
-    << endl;
+  o << "EXTRA_CFLAGS += -Wno-unused " << WERROR << endl;
   #if CHECK_POINTER_ARITH_PR5947
   o << "EXTRA_CFLAGS += -Wpointer-arith" << endl;
   #endif
 
+  // Accept extra diagnostic-suppression pragmas etc.
+  o << "EXTRA_CFLAGS += -Wno-pragmas" << endl;
+
+  // PR25845: Recent gcc (seen on 9.3.1) warns fairly common 32-bit pointer-conversions:
+  o << "EXTRA_CFLAGS += $(call cc-option,-Wno-pointer-to-int-cast)" << endl;
+  o << "EXTRA_CFLAGS += $(call cc-option,-Wno-int-to-pointer-cast)" << endl;
+  // TODO: Some tests also suffer from -Werror=overflow. That seems like a warning requiring a tiny bit more care.
+
   // If we've got a reasonable runtime path from the user, we'll just
   // do '-IDIR'. If there are any sneaky/odd characters in it, we'll
   // have to quote it, like '-I"DIR"'.
@@ -567,12 +638,21 @@ compile_pass (systemtap_session& s)
       objname[objname.size()-1] = 'o'; // now objname
       o << " " + objname;
     }
-  o << endl;
+  o << " stap_symbols.o" << endl;
+
+  o << s.tmpdir << "/stap_symbols.o: $(STAPCONF_HEADER)" << endl;
 
   // add all stapconf dependencies
-  o << s.translated_source << ": $(STAPCONF_HEADER)" << endl;
-  for (unsigned i=0; i<s.auxiliary_outputs.size(); i++)
-    o << s.auxiliary_outputs[i]->filename << ": $(STAPCONF_HEADER)" << endl;  
+  string translated = s.translated_source;
+  translated[translated.size()-1] = 'o';
+  o << translated << ": $(STAPCONF_HEADER)" << endl;
+  translated[translated.size()-1] = 'i';
+  o << translated << ": $(STAPCONF_HEADER)" << endl;
+  for (unsigned i=0; i<s.auxiliary_outputs.size(); i++) {
+    translated = s.auxiliary_outputs[i]->filename;
+    translated[translated.size()-1] = 'o';
+    o << translated << ": $(STAPCONF_HEADER)" << endl;
+  }
 
   o.close ();
 
@@ -876,6 +956,9 @@ make_bpf_run_command (systemtap_session& s, const string& remotedir,
   if (s.suppress_warnings)
     cmd.push_back("-w");
 
+  if (s.target_pid)
+    cmd.insert(cmd.end(), { "-x", lex_cast(s.target_pid) });
+
   if (!s.output_file.empty())
     {
       cmd.push_back("-o");
@@ -1062,9 +1145,10 @@ make_typequery_umod(systemtap_session& s, const vector<string>& headers, string&
   // cwd in this case will be the cwd of stap itself though, which may be
   // trickier to deal with.  It might be better to "cd `dirname $script`"
   // first...
+  // s.runtime_path allows finding linux/stp_tls.h
   vector<string> cmd
     {
-      "gcc", "-shared", "-g", "-fno-eliminate-unused-debug-types",
+      "gcc", "-shared", "-g", "-I", s.runtime_path, "-fno-eliminate-unused-debug-types",
       "-xc", "/dev/null", "-o", name,
     };
   for (size_t i = 0; i < headers.size(); ++i)
