@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2014 Red Hat Inc.
+// Copyright (C) 2014-2018 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -30,6 +30,7 @@ enum {
   LONG_OPT_USE_SERVER,
   LONG_OPT_LIST_SERVERS,
   LONG_OPT_TRUST_SERVERS,
+  LONG_OPT_USE_HTTP_SERVER,
   LONG_OPT_ALL_MODULES,
   LONG_OPT_REMOTE,
   LONG_OPT_CHECK_VERSION,
@@ -54,6 +55,7 @@ enum {
   LONG_OPT_SUPPRESS_TIME_LIMITS,
   LONG_OPT_RUNTIME,
   LONG_OPT_RUNTIME_DYNINST,
+  LONG_OPT_RUNTIME_BPF,
   LONG_OPT_BENCHMARK_SDT,
   LONG_OPT_BENCHMARK_SDT_LOOPS,
   LONG_OPT_BENCHMARK_SDT_THREADS,
@@ -63,6 +65,8 @@ enum {
   LONG_OPT_TARGET_NAMESPACES,
   LONG_OPT_MONITOR,
   LONG_OPT_INTERACTIVE,
+  LONG_OPT_RUN_EXAMPLE,
+  LONG_OPT_NO_GLOBAL_VAR_DISPLAY,
 };
 
 // NB: when adding new options, consider very carefully whether they
