@@ -1,5 +1,5 @@
 // systemtap translator/driver
-// Copyright (C) 2005-2018 Red Hat Inc.
+// Copyright (C) 2005-2019 Red Hat Inc.
 // Copyright (C) 2005 IBM Corp.
 // Copyright (C) 2006 Intel Corporation.
 //
@@ -54,6 +54,7 @@ extern "C" {
 #include <unistd.h>
 #include <wordexp.h>
 #include <ftw.h>
+#include <fcntl.h>
 }
 
 using namespace std;
@@ -134,11 +135,13 @@ printscript(systemtap_session& s, ostream& o)
       // print probe name and variables if there
       for (map<string, set<derived_probe *> >::iterator it=probe_list.begin(); it!=probe_list.end(); ++it)
         {
+          std::string pp;
+
           // probe name or alias
           if (s.dump_mode == systemtap_session::dump_matched_probes_vars && isatty(STDOUT_FILENO))
-            o << s.colorize(it->first, "source");
+            pp = s.colorize(it->first, "source");
           else
-            o << it->first;
+            pp = it->first;
 
           // Print the locals and arguments for -L mode only
           if (s.dump_mode == systemtap_session::dump_matched_probes_vars)
@@ -167,20 +170,55 @@ printscript(systemtap_session& s, ostream& o)
                     arg_count[*ia]++;
                     arg_list.push_back(*ia);
 		  }
+
+                  // If verbosity is set, we want to print out all instances of duplicated probes and
+                  // distinguish them with the PC address.
+                  if (s.verbose > 0)
+                    {
+                      // We want to print the probe point signature (without the nested components).
+                      std::ostringstream sig;
+                      p->printsig(sig, false);
+
+                      if (s.dump_mode == systemtap_session::dump_matched_probes_vars && isatty(STDOUT_FILENO))
+                        o << s.colorize(sig.str(), "source");
+                      else
+                        o << sig.str();
+
+                      for (list<string>::iterator ir=var_list.begin(); ir!=var_list.end(); ++ir)
+                        o << " " << *ir;
+                      for (list<string>::iterator ir=arg_list.begin(); ir!=arg_list.end(); ++ir)
+                        o << " " << *ir;
+                     
+                      o << endl;
+
+                      arg_list.clear();
+                      var_list.clear();
+                    }
                 }
 
-	      uniq_list(arg_list);
-	      uniq_list(var_list);
+              if (!(s.verbose > 0))
+                {
+                  uniq_list(arg_list);
+                  uniq_list(var_list);
+                  
+                  o << pp;
 
-              // print the set-intersection only
-              for (list<string>::iterator ir=var_list.begin(); ir!=var_list.end(); ++ir)
-                if (var_count.find(*ir)->second == it->second.size()) // print locals
-                  o << " " << *ir;
-              for (list<string>::iterator ir=arg_list.begin(); ir!=arg_list.end(); ++ir)
-                if (arg_count.find(*ir)->second == it->second.size()) // print arguments
-                  o << " " << *ir;
+                  // print the set-intersection only
+                  for (list<string>::iterator ir=var_list.begin(); ir!=var_list.end(); ++ir)
+                    if (var_count.find(*ir)->second == it->second.size()) // print locals
+                      o << " " << *ir;
+                  for (list<string>::iterator ir=arg_list.begin(); ir!=arg_list.end(); ++ir)
+                    if (arg_count.find(*ir)->second == it->second.size()) // print arguments
+                      o << " " << *ir;
+
+                  o << endl;
+                }
             }
-          o << endl;
+          else
+            { 
+              o << pp;
+              o << endl;
+            } 
         }
     }
   else
@@ -229,8 +267,9 @@ printscript(systemtap_session& s, ostream& o)
             }
           if (s.verbose)
             {
+              o << "{ ";
               f->body->print (o);
-              o << endl;
+              o << " }" << endl;
             }
         }
 
@@ -253,8 +292,9 @@ printscript(systemtap_session& s, ostream& o)
             }
           if (s.verbose)
             {
+              o << "{ ";
               p->body->print (o);
-              o << endl;
+              o << " }" << endl;
             }
         }
     }
@@ -277,8 +317,30 @@ void handle_interrupt (int)
   if (pending_interrupts > 2) // XXX: should be configurable? time-based?
     {
       char msg[] = "Too many interrupts received, exiting.\n";
-      int rc = write (2, msg, sizeof(msg)-1);
-      if (rc) {/* Do nothing; we don't care if our last gasp went out. */ ;}
+      int fd = 2;
+
+      /* NB: writing to stderr blockingly in a signal handler is dangerous
+       * since it may prevent the stap process from quitting gracefully
+       * on receiving SIGTERM/etc signals when the stderr write buffer
+       * is full. PR23891 */
+      int flags = fcntl(fd, F_GETFL);
+      if (flags == -1)
+        _exit (1);
+
+      if (!(flags & O_NONBLOCK))
+        {
+          if (fcntl(fd, F_SETFL, flags | O_NONBLOCK) == 0) {
+            int rc = write (fd, msg, sizeof(msg)-1);
+            if (rc)
+              {
+                /* Do nothing; we don't care if our last gasp went out. */
+                ;
+              }
+          }
+        }  /* if ! O_NONBLOCK */
+
+      /* to avoid leaving any side-effects on the stderr device */
+      (void) fcntl(fd, F_SETFL, flags & ~O_NONBLOCK);
       _exit (1);
     }
 }
@@ -313,9 +375,10 @@ setup_signals (sighandler_t handler)
 
 
 static void
-sdt_benchmark_thread(unsigned long i)
+sdt_benchmark_thread(unsigned long i, double *fpointer)
 {
   PROBE(stap, benchmark__thread__start);
+  *fpointer += 0.0;
   while (i--)
     PROBE1(stap, benchmark, i);
   PROBE(stap, benchmark__thread__end);
@@ -327,7 +390,7 @@ run_sdt_benchmark(systemtap_session& s)
 {
   unsigned long loops = s.benchmark_sdt_loops ?: 10000000;
   unsigned long threads = s.benchmark_sdt_threads ?: 1;
-
+  
   if (s.verbose > 0)
     clog << _F("Beginning SDT benchmark with %lu loops in %lu threads.",
                loops, threads) << endl;
@@ -341,8 +404,9 @@ run_sdt_benchmark(systemtap_session& s)
   PROBE(stap, benchmark__start);
     {
       vector<thread> handles;
+      double f = 2.71828;
       for (unsigned long i = 0; i < threads; ++i)
-        handles.push_back(thread(sdt_benchmark_thread, loops));
+        handles.push_back(thread(sdt_benchmark_thread, loops,&f));
       for (unsigned long i = 0; i < threads; ++i)
         handles[i].join();
     }
@@ -477,11 +541,16 @@ passes_0_4 (systemtap_session &s)
 	}
     }
 
-  // Create the name of the C source file within the temporary
+  // Create the name of the main C source file within the temporary
   // directory.  Note the _src prefix, explained in
   // buildrun.cxx:compile_pass()
   s.translated_source = string(s.tmpdir) + "/" + s.module_name + "_src.c";
 
+  // Create the name of the C source file for the dumped symbol data
+  // within the temporary directory. This file will be generated in
+  // translate.cxx:emit_symbol_data()
+  s.symbols_source = string(s.tmpdir) + "/stap_symbols.c";
+
   PROBE1(stap, pass0__end, &s);
 
   struct tms tms_before;
@@ -1082,13 +1151,28 @@ passes_0_4 (systemtap_session &s)
 
   // PASS 3: TRANSLATION
   // (BPF does translation and compilation at once in pass 4.)
-  if (s.runtime_mode != systemtap_session::bpf_runtime)
-    {
-      s.verbose = s.perpass_verbose[2];
-      times (& tms_before);
-      gettimeofday (&tv_before, NULL);
-      PROBE1(stap, pass3__start, &s);
+  s.verbose = s.perpass_verbose[2];
+  times (& tms_before);
+  gettimeofday (&tv_before, NULL);
+  PROBE1(stap, pass3__start, &s);
 
+  if (s.runtime_mode == systemtap_session::bpf_runtime)
+    {
+      times (& tms_after);
+      gettimeofday (&tv_after, NULL);
+      PROBE1(stap, pass3__end, &s);
+
+      if (s.verbose)
+	clog << _("Pass 3: pass skipped for stapbpf runtime ")
+	     << TIMESPRINT
+	     << endl;
+
+      assert_no_interrupts();
+      if (s.last_pass == 3)
+	return rc;
+    }
+  else
+    {
       rc = translate_pass (s);
 
       if (! rc && s.last_pass == 3)
@@ -1391,6 +1475,9 @@ main (int argc, char * const argv [])
             if (ss.verbose > 1)
 	      clog << _F("Session arch: %s release: %s",
 			 ss.architecture.c_str(), ss.kernel_release.c_str())
+		   << endl
+                   << _F("Build tree: \"%s\"",
+			 ss.kernel_build_tree.c_str())
 		   << endl;
 
 #if HAVE_NSS
