@@ -503,7 +503,7 @@ add_server_trust (
   // Shutdown NSS.
   // SSL_ClearSessionCache is required before shutdown for client applications.
   SSL_ClearSessionCache ();
-  nssCleanup (cert_db_path.c_str ());
+  nssCleanup (cert_db_path.c_str (), NULL);
 
   // Make sure the database files are readable.
   glob_t globbuf;
@@ -559,12 +559,13 @@ revoke_server_trust (
   CERTCertificate *db_cert;
   vector<string> processed_certs;
   const char *nickname;
-
+  SECStatus secStatus;
+  
   // Make sure NSPR is initialized. Must be done before NSS is initialized
   s.NSPR_init ();
 
   // Initialize the NSS libraries -- read/write
-  SECStatus secStatus = nssInit (cert_db_path.c_str (), 1/*readwrite*/);
+  secStatus = nssInit (cert_db_path.c_str (), 1/*readwrite*/);
   if (secStatus != SECSuccess)
     {
       // Message already issued
@@ -672,7 +673,7 @@ revoke_server_trust (
   if (tmpArena)
     PORT_FreeArena (tmpArena, PR_FALSE);
 
-  nssCleanup (cert_db_path.c_str ());
+  nssCleanup (cert_db_path.c_str (), NULL);
 }
 
 // Obtain information about servers from the certificates in the given database.
@@ -683,6 +684,8 @@ get_server_info_from_db (
     const string &cert_db_path
   )
 {
+  NSSInitContext *context;
+
   // Make sure the given path exists.
   if (! file_exists (cert_db_path))
     {
@@ -696,8 +699,8 @@ get_server_info_from_db (
   s.NSPR_init ();
 
   // Initialize the NSS libraries -- readonly
-  SECStatus secStatus = nssInit (cert_db_path.c_str ());
-  if (secStatus != SECSuccess)
+  context = nssInitContext (cert_db_path.c_str (), 0 /* readWrite */, 0 /* issueMessage */);
+  if (context == NULL)
     {
       // Message already issued.
       return;
@@ -741,7 +744,7 @@ get_server_info_from_db (
   if (certs)
     CERT_DestroyCertList (certs);
 
-  nssCleanup (cert_db_path.c_str ());
+  nssCleanup (cert_db_path.c_str (), context);
 }
 
 // Utility Functions.
@@ -1816,6 +1819,8 @@ nss_get_or_keep_compatible_server_info (
 	  && servers[i].sysinfo != s.architecture)
 	{
 	  // Target platform mismatch.
+	  if (s.verbose > 3)
+	    clog << "Rejected server (platform): " << servers[i] << endl;
 	  servers.erase (servers.begin () + i);
 	  continue;
 	}
@@ -1824,20 +1829,14 @@ nss_get_or_keep_compatible_server_info (
       // server has the right MOK.
       if (! s.mok_fingerprints.empty ())
         {
-	  // This server has no MOKs.
-	  if (servers[i].mok_fingerprints.empty ())
-	    {
-	      servers.erase (servers.begin () + i);
-	      continue;
-	    }
-
 	  // Make sure the server has at least one MOK in common with
 	  // the client.
 	  vector<string>::const_iterator it;
 	  bool mok_found = false;
 	  for (it = s.mok_fingerprints.begin(); it != s.mok_fingerprints.end(); it++)
 	    {
-	      if (find(servers[i].mok_fingerprints.begin(),
+	      if ((*it == "missing") || // PR26665: see also session.cxx check_options()
+		  find(servers[i].mok_fingerprints.begin(),
 		       servers[i].mok_fingerprints.end(), *it)
 		  != servers[i].mok_fingerprints.end ())
 	        {
@@ -1849,12 +1848,16 @@ nss_get_or_keep_compatible_server_info (
 	  // This server has no MOK in common with the client.
 	  if (! mok_found)
 	    {
+	      if (s.verbose > 3)
+		clog << "Rejected server (common mok): " << servers[i] << endl;
 	      servers.erase (servers.begin () + i);
 	      continue;
 	    }
 	}
 
       // The server is compatible. Leave it in the list.
+      if (s.verbose > 3)
+	clog << "Accepted server: " << servers[i] << endl;
       ++i;
     }
 #else // ! HAVE_AVAHI
