@@ -85,7 +85,14 @@ class direct : public remote {
       {
         args = make_run_command(*s);
         if (! staprun_r_arg.empty()) // PR13354
-          args.insert(args.end(), { "-r", staprun_r_arg });
+          {
+            if (s->runtime_mode == systemtap_session::dyninst_runtime)
+              args.insert(args.end(), {"_stp_dyninst_remote=" + staprun_r_arg});
+	    else if (s->runtime_mode == systemtap_session::kernel_runtime)
+              args.insert(args.end(), { "-r", staprun_r_arg });
+	    // leave args empty for bpf_runtime
+          }
+
         pid_t pid = stap_spawn (s->verbose, args);
         if (pid <= 0)
           return 1;
@@ -445,8 +452,17 @@ class stapsh : public remote {
       {
         int rc = 0;
 
-        string localmodule = s->tmpdir + "/" + s->module_name + ".ko";
-        string remotemodule = s->module_name + ".ko";
+        string extension;
+        if (s->runtime_mode == systemtap_session::dyninst_runtime)
+	  extension = ".so";
+	else if (s->runtime_mode == systemtap_session::bpf_runtime)
+	  extension = ".bo";
+	else
+	  extension = ".ko";
+
+	string localmodule = s->tmpdir + "/" + s->module_name + extension;
+        string remotemodule = s->module_name + extension;
+
         if ((rc = send_file(localmodule, remotemodule)))
           return rc;
 
@@ -478,7 +494,13 @@ class stapsh : public remote {
         // PR13354: identify our remote index/url
         if (strverscmp("1.7", remote_version.c_str()) <= 0 && // -r supported?
             ! staprun_r_arg.empty())
-          cmd.insert(cmd.end(), { "-r", staprun_r_arg });
+          {
+             if (s->runtime_mode == systemtap_session::dyninst_runtime)
+	       cmd.insert(cmd.end(), { "_stp_dyninst_remote=" +  staprun_r_arg});
+             else if (s->runtime_mode == systemtap_session::kernel_runtime)
+               cmd.insert(cmd.end(), { "-r", staprun_r_arg });
+	     // leave args empty for bpf_runtime
+          }
 
         for (unsigned i = 1; i < cmd.size(); ++i)
           run << ' ' << qpencode(cmd[i]);
