@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Memory allocation functions
- * Copyright (C) 2005-2008 Red Hat Inc.
+ * Copyright (C) 2005-2018 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -21,6 +21,13 @@ static int _stp_allocated_net_memory = 0;
    suppress the oom-killer from kicking in. */
 #define STP_ALLOC_SLEEP_FLAGS (GFP_KERNEL | __GFP_NORETRY)
 
+/* #define DEBUG_MEMALLOC_MIGHT_SLEEP */
+/*
+ * If DEBUG_MEMALLOC_MIGHT_SLEEP is defined (stap -DDEBUG_MEMALLOC_MIGHT_SLEEP ...)
+ * then all memory allocation operations are assumed to sleep. This
+ * can be used to check whether any memory allocations occur in atomic
+ * context, which is discouraged on realtime kernels. */
+
 /* #define DEBUG_MEM */
 /*
  * If DEBUG_MEM is defined (stap -DDEBUG_MEM ...) then full memory
@@ -52,7 +59,7 @@ typedef struct {
 	char *free;
 } _stp_malloc_type;
 
-static const _stp_malloc_type const _stp_malloc_types[] = {
+static const _stp_malloc_type _stp_malloc_types[] = {
 	{"kmalloc", "kfree"},
 	{"vmalloc", "vfree"},
 	{"alloc_percpu", "free_percpu"}
@@ -99,6 +106,7 @@ static void *_stp_mem_debug_setup(void *addr, size_t size, enum _stp_memtype typ
 {
 	struct list_head *p;
 	struct _stp_mem_entry *m;
+	unsigned long flags;
 	memset(addr, 0x55, MEM_FENCE_SIZE);
 	addr += MEM_FENCE_SIZE;
 	memset(addr + size, 0x55, MEM_FENCE_SIZE);
@@ -108,9 +116,9 @@ static void *_stp_mem_debug_setup(void *addr, size_t size, enum _stp_memtype typ
 	m->type = type;
 	m->len = size;
 	m->addr = addr;
-	spin_lock(&_stp_mem_lock);
+	spin_lock_irqsave(&_stp_mem_lock, flags);
 	list_add(p, &_stp_mem_list); 
-	spin_unlock(&_stp_mem_lock);
+	spin_unlock_irqrestore(&_stp_mem_lock, flags);
 	return addr;
 }
 
@@ -118,13 +126,14 @@ static void *_stp_mem_debug_setup(void *addr, size_t size, enum _stp_memtype typ
 static void _stp_mem_debug_percpu(struct _stp_mem_entry *m, void *addr, size_t size)
 {
 	struct list_head *p = (struct list_head *)m;
+	unsigned long flags;
 	m->magic = MEM_MAGIC;
 	m->type = MEM_PERCPU;
 	m->len = size;
 	m->addr = addr;
-	spin_lock(&_stp_mem_lock);
+	spin_lock_irqsave(&_stp_mem_lock, flags);
 	list_add(p, &_stp_mem_list);
-	spin_unlock(&_stp_mem_lock);	
+	spin_unlock_irqrestore(&_stp_mem_lock, flags);
 }
 
 static void _stp_mem_debug_free(void *addr, enum _stp_memtype type)
@@ -132,8 +141,9 @@ static void _stp_mem_debug_free(void *addr, enum _stp_memtype type)
 	int found = 0;
 	struct list_head *p, *tmp;
 	struct _stp_mem_entry *m = NULL;
+	unsigned long flags;
 
-	spin_lock(&_stp_mem_lock);
+	spin_lock_irqsave(&_stp_mem_lock, flags);
 	list_for_each_safe(p, tmp, &_stp_mem_list) {
 		m = list_entry(p, struct _stp_mem_entry, list);
 		if (m->addr == addr) {
@@ -142,7 +152,7 @@ static void _stp_mem_debug_free(void *addr, enum _stp_memtype type)
 			break;
 		}
 	}
-	spin_unlock(&_stp_mem_lock);
+	spin_unlock_irqrestore(&_stp_mem_lock, flags);
 	if (!found) {
 		printk("SYSTEMTAP ERROR: Free of unallocated memory %p type=%s\n", 
 		       addr, _stp_malloc_types[type].free);
@@ -183,8 +193,9 @@ static void _stp_mem_debug_validate(void *addr)
 	int found = 0;
 	struct list_head *p, *tmp;
 	struct _stp_mem_entry *m = NULL;
+	unsigned long flags;
 
-	spin_lock(&_stp_mem_lock);
+	spin_lock_irqsave(&_stp_mem_lock, flags);
 	list_for_each_safe(p, tmp, &_stp_mem_list) {
 		m = list_entry(p, struct _stp_mem_entry, list);
 		if (m->addr == addr) {
@@ -192,7 +203,7 @@ static void _stp_mem_debug_validate(void *addr)
 			break;
 		}
 	}
-	spin_unlock(&_stp_mem_lock);
+	spin_unlock_irqrestore(&_stp_mem_lock, flags);
 	if (!found) {
 		printk("SYSTEMTAP ERROR: Couldn't validate memory %p\n", 
 		       addr);
@@ -259,6 +270,9 @@ static void _stp_mem_debug_validate(void *addr)
 static void *_stp_kmalloc_gfp(size_t size, gfp_t gfp_mask)
 {
 	void *ret;
+#ifdef DEBUG_MEMALLOC_MIGHT_SLEEP
+	might_sleep();
+#endif
 #ifdef STP_MAXMEMORY
 	if ((_STP_MODULE_CORE_SIZE + _stp_allocated_memory + size)
 	    > (STP_MAXMEMORY * 1024)) {
@@ -472,6 +486,9 @@ static void *_stp_alloc_percpu(size_t size)
 static void *_stp_kmalloc_node_gfp(size_t size, int node, gfp_t gfp_mask)
 {
 	void *ret;
+#ifdef DEBUG_MEMALLOC_MIGHT_SLEEP
+	might_sleep();
+#endif
 #ifdef STP_MAXMEMORY
 	if ((_STP_MODULE_CORE_SIZE + _stp_allocated_memory + size)
 	    > (STP_MAXMEMORY * 1024)) {
@@ -505,6 +522,9 @@ static void *_stp_kzalloc_node_gfp(size_t size, int node, gfp_t gfp_mask)
 	 * So for now, just malloc and zero it manually.
 	 */
 	void *ret = _stp_kmalloc_node_gfp(size, node, gfp_mask);
+#ifdef DEBUG_MEMALLOC_MIGHT_SLEEP
+	might_sleep();
+#endif
 	if (likely(ret)) {
 		memset (ret, 0, size);
 	}
@@ -522,6 +542,9 @@ static void *_stp_kzalloc_node(size_t size, int node)
 
 static void _stp_kfree(void *addr)
 {
+#ifdef DEBUG_MEMALLOC_MIGHT_SLEEP
+	might_sleep();
+#endif
 #ifdef DEBUG_MEM
 	_stp_mem_debug_free(addr, MEM_KMALLOC);
 #else
@@ -531,6 +554,9 @@ static void _stp_kfree(void *addr)
 
 static void _stp_vfree(void *addr)
 {
+#ifdef DEBUG_MEMALLOC_MIGHT_SLEEP
+	might_sleep();
+#endif
 #ifdef DEBUG_MEM
 	_stp_mem_debug_free(addr, MEM_VMALLOC);
 #else
@@ -540,6 +566,9 @@ static void _stp_vfree(void *addr)
 
 static void _stp_free_percpu(void *addr)
 {
+#ifdef DEBUG_MEMALLOC_MIGHT_SLEEP
+	might_sleep();
+#endif
 #ifdef DEBUG_MEM
 	_stp_mem_debug_free(addr, MEM_PERCPU);
 #else
@@ -552,8 +581,9 @@ static void _stp_mem_debug_done(void)
 #ifdef DEBUG_MEM
 	struct list_head *p, *tmp;
 	struct _stp_mem_entry *m;
+	unsigned long flags;
 
-	spin_lock(&_stp_mem_lock);
+	spin_lock_irqsave(&_stp_mem_lock, flags);
 	list_for_each_safe(p, tmp, &_stp_mem_list) {
 		m = list_entry(p, struct _stp_mem_entry, list);
 		list_del(p);
@@ -585,7 +615,7 @@ static void _stp_mem_debug_done(void)
 		}
 	}
 done:
-	spin_unlock(&_stp_mem_lock);
+	spin_unlock_irqrestore(&_stp_mem_lock, flags);
 
 	return;
 
