@@ -16,6 +16,27 @@
 #error "Need CONFIG_KPROBES!"
 #endif
 
+// PR26074: if kallsyms_on_each_symbol() is not exported we try to get it via relocations.
+// Need to define the runtime.h variable here to check if it's defined from kprobes.c.
+// (This code would go in a header if it were used anywhere else.)
+//
+// PR11514: kallsyms_on_each_symbol() should not be used on PPC64.
+#ifndef CONFIG_PPC64
+#define STAPCONF_KALLSYMS_ON_EACH_SYMBOL
+#if !defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL_EXPORTED)
+extern void *_stp_kallsyms_on_each_symbol;
+#endif
+#endif
+
+#if defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL) && defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL_EXPORTED)
+#define USE_KALLSYMS_ON_EACH_SYMBOL (1)
+#elif defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL)
+// XXX May not be available; if available, is passed through relocation:
+#define USE_KALLSYMS_ON_EACH_SYMBOL (_stp_kallsyms_on_each_symbol != NULL)
+#else
+#define USE_KALLSYMS_ON_EACH_SYMBOL (0)
+#endif
+
 #include <linux/kprobes.h>
 #include <linux/module.h>
 
@@ -136,41 +157,40 @@ stapkp_prepare_kprobe(struct stap_kprobe_probe *skp)
       kp->addr = (void *) addr;
    }
    else {
-#ifdef STAPCONF_KALLSYMS_ON_EACH_SYMBOL
       // If we're doing symbolic name + offset probing (that gets
       // converted to an address), it doesn't really matter if the
       // symbol is in a module and the module isn't loaded right
       // now. The registration will fail, but will get tried again
       // when the module is loaded.
-      if (kp->addr == 0)
+      if (USE_KALLSYMS_ON_EACH_SYMBOL && kp->addr == 0)
 	 return 1;
-#else
-      // If we don't have kallsyms_on_each_symbol(), we'll use
-      // symbol_name+offset probing and let
-      // register_kprobe()/register_kretprobe() call
-      // kallsyms_lookup_name() for us. However, on kernels < 3.11,
-      // module_kallsyms_lookup_name() (called from
-      // kallsyms_lookup_name()) has a bug where it modifies its
-      // argument. So, for those kernels we'll workaround the bug by
-      // duplicating the string (so we go from read-only memory in the
-      // initialized struct data to read-write allocated memory). The
-      // memory gets freed when the probe is unregistered.
-      //
-      // This bug was fixed in kernel 3.11+ by the following commit:
-      //
-      //   commit 4f6de4d51f4a3ab06a85e91e708cc89a513ef30c
-      //      Author: Mathias Krause <minipli@googlemail.com>
-      //      Date:   Tue Jul 2 15:35:11 2013 +0930
-      //
-      //      module: don't modify argument of module_kallsyms_lookup_name()
+      else if (!USE_KALLSYMS_ON_EACH_SYMBOL) {
+        // If we don't have kallsyms_on_each_symbol(), we'll use
+        // symbol_name+offset probing and let
+        // register_kprobe()/register_kretprobe() call
+        // kallsyms_lookup_name() for us. However, on kernels < 3.11,
+        // module_kallsyms_lookup_name() (called from
+        // kallsyms_lookup_name()) has a bug where it modifies its
+        // argument. So, for those kernels we'll workaround the bug by
+        // duplicating the string (so we go from read-only memory in the
+        // initialized struct data to read-write allocated memory). The
+        // memory gets freed when the probe is unregistered.
+        //
+        // This bug was fixed in kernel 3.11+ by the following commit:
+        //
+        //   commit 4f6de4d51f4a3ab06a85e91e708cc89a513ef30c
+        //      Author: Mathias Krause <minipli@googlemail.com>
+        //      Date:   Tue Jul 2 15:35:11 2013 +0930
+        //
+        //      module: don't modify argument of module_kallsyms_lookup_name()
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0)
-      if (kp->symbol_name == NULL)
-	 kp->symbol_name = kstrdup(skp->symbol_name, STP_ALLOC_FLAGS);
+        if (kp->symbol_name == NULL)
+          kp->symbol_name = kstrdup(skp->symbol_name, STP_ALLOC_FLAGS);
 #else
-      kp->symbol_name = (typeof(kp->symbol_name))skp->symbol_name;
-#endif
-      kp->offset = skp->offset;
+        kp->symbol_name = (typeof(kp->symbol_name))skp->symbol_name;
 #endif
+        kp->offset = skp->offset;
+      }
    }
 
    kp->pre_handler = &enter_kprobe_probe;
@@ -247,18 +267,17 @@ stapkp_prepare_kretprobe(struct stap_kprobe_probe *skp)
       krp->kp.addr = (void *) addr;
    }
    else {
-#ifdef STAPCONF_KALLSYMS_ON_EACH_SYMBOL
-      if (krp->kp.addr == 0)
+      if (USE_KALLSYMS_ON_EACH_SYMBOL && krp->kp.addr == 0)
 	 return 1;
-#else
+      else if (!USE_KALLSYMS_ON_EACH_SYMBOL) {
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0)
-      if (krp->kp.symbol_name == NULL)
-	 krp->kp.symbol_name = kstrdup(skp->symbol_name, STP_ALLOC_FLAGS);
+        if (krp->kp.symbol_name == NULL)
+          krp->kp.symbol_name = kstrdup(skp->symbol_name, STP_ALLOC_FLAGS);
 #else
-      krp->kp.symbol_name = (typeof(krp->kp.symbol_name))skp->symbol_name;
-#endif
-      krp->kp.offset = skp->offset;
+        krp->kp.symbol_name = (typeof(krp->kp.symbol_name))skp->symbol_name;
 #endif
+        krp->kp.offset = skp->offset;
+      }
    }
 
    if (skp->maxactive_p)
@@ -641,7 +660,6 @@ stapkp_refresh_probe(struct stap_kprobe_probe *skp)
 #endif /* LINUX_VERSION_CODE >= 2.6.30 */
 
 
-#ifdef STAPCONF_KALLSYMS_ON_EACH_SYMBOL
 struct stapkp_symbol_data {
    struct stap_kprobe_probe *probes;
    size_t nprobes;			/* number of probes in "probes" */
@@ -699,7 +717,6 @@ stapkp_symbol_callback(void *data, const char *name,
    }
    return 0;
 }
-#endif
 
 
 static int
@@ -708,32 +725,34 @@ stapkp_init(struct stap_kprobe_probe *probes,
 {
    size_t i;
 
-#ifdef STAPCONF_KALLSYMS_ON_EACH_SYMBOL
-   // If we have any symbol_name+offset probes, we need to try to
-   // convert those into address-based probes.
-   size_t probe_max = 0;
-   for (i = 0; i < nprobes; i++) {
-      struct stap_kprobe_probe *skp = &probes[i];
+   if (USE_KALLSYMS_ON_EACH_SYMBOL) {
+     // If we have any symbol_name+offset probes, we need to try to
+     // convert those into address-based probes.
+     size_t probe_max = 0;
+     for (i = 0; i < nprobes; i++) {
+       struct stap_kprobe_probe *skp = &probes[i];
 
-      if (! skp->symbol_name)
+       if (! skp->symbol_name)
 	 continue;
-      ++probe_max;
+       ++probe_max;
+     }
+     if (probe_max > 0) {
+       // Here we're going to try to convert any symbol_name+offset
+       // probes into address probes.
+       struct stapkp_symbol_data sd;
+       dbug_stapkp("looking up %lu probes\n", probe_max);
+       sd.probes = probes;
+       sd.nprobes = nprobes;
+       sd.probe_max = probe_max;
+       sd.modname = NULL;
+       mutex_lock(&module_mutex);
+       preempt_disable();
+       kallsyms_on_each_symbol(stapkp_symbol_callback, &sd);
+       preempt_enable();
+       mutex_unlock(&module_mutex);
+       dbug_stapkp("found %lu probes\n", sd.probe_max);
+     }
    }
-   if (probe_max > 0) {
-      // Here we're going to try to convert any symbol_name+offset
-      // probes into address probes.
-      struct stapkp_symbol_data sd;
-      dbug_stapkp("looking up %lu probes\n", probe_max);
-      sd.probes = probes;
-      sd.nprobes = nprobes;
-      sd.probe_max = probe_max;
-      sd.modname = NULL;
-      preempt_disable();
-      kallsyms_on_each_symbol(stapkp_symbol_callback, &sd);
-      preempt_enable();
-      dbug_stapkp("found %lu probes\n", sd.probe_max);
-   }
-#endif
 
    for (i = 0; i < nprobes; i++) {
       struct stap_kprobe_probe *skp = &probes[i];
@@ -773,10 +792,10 @@ stapkp_refresh(const char *modname,
 
    dbug_stapkp("refresh %lu probes with module %s\n", nprobes, modname ?: "?");
 
-#ifdef STAPCONF_KALLSYMS_ON_EACH_SYMBOL
-   if (modname) {
-      size_t probe_max = 0;
-      for (i = 0; i < nprobes; i++) {
+   if (USE_KALLSYMS_ON_EACH_SYMBOL) {
+     if (modname) {
+       size_t probe_max = 0;
+       for (i = 0; i < nprobes; i++) {
 	 struct stap_kprobe_probe *skp = &probes[i];
 
 	 // If this probe is in the same module that is being
@@ -786,20 +805,22 @@ stapkp_refresh(const char *modname,
 	 // address-based probes.
 	 if (skp->module && strcmp(modname, skp->module) == 0
 	     && skp->symbol_name && skp->registered_p == 0)
-	    ++probe_max;
-      }
-      if (probe_max > 0) {
+           ++probe_max;
+       }
+       if (probe_max > 0) {
 	 struct stapkp_symbol_data sd;
 	 sd.probes = probes;
 	 sd.nprobes = nprobes;
 	 sd.probe_max = probe_max;
 	 sd.modname = modname;
+	 mutex_lock(&module_mutex);
 	 preempt_disable();
 	 kallsyms_on_each_symbol(stapkp_symbol_callback, &sd);
 	 preempt_enable();
-      }
+	 mutex_unlock(&module_mutex);
+       }
+     }
    }
-#endif
 
    for (i = 0; i < nprobes; i++) {
 
