@@ -28,9 +28,13 @@
 #include <linux/hardirq.h>
 #include <asm/uaccess.h>
 #include <linux/kallsyms.h>
-#include <linux/vermagic.h>
-#include <linux/utsname.h>
 #include <linux/version.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,17,0)
+#include <generated/utsrelease.h>
+#else
+#include <linux/vermagic.h>
+#endif
+#include <linux/utsname.h>
 #include <linux/compat.h>
 #include <linux/sched.h>
 #include <linux/mm.h>
@@ -44,23 +48,30 @@
 #include <generated/compile.h>
 #endif
 
+// PR26811: Replace some declarations after set_fs() removal in kernel 5.10+.
+// Should use the STP_* prefixed defines outside of STAPCONF_SET_FS.
+#if defined(STAPCONF_SET_FS)
+#define stp_mm_segment_t mm_segment_t
+#define STP_KERNEL_DS KERNEL_DS
+#define STP_USER_DS USER_DS
+// XXX MM_SEG_IS_KERNEL only used #ifndef STAPCONF_SET_FS
+#else
+#define stp_mm_segment_t unsigned long
+#define STP_KERNEL_DS 0
+#define STP_USER_DS 1
+#define MM_SEG_IS_KERNEL(seg) ((seg)==STP_KERNEL_DS)
+
+// Required for kernel write operations:
+static void *kallsyms_copy_to_kernel_nofault;
+#endif
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
 #include <linux/user_namespace.h>
 #endif
 
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,15)
 #if !defined (CONFIG_DEBUG_FS)  && !defined (CONFIG_DEBUG_FS_MODULE)
 #error "DebugFS is required and was not found in the kernel."
 #endif
-#ifdef CONFIG_RING_BUFFER
-#define STP_TRANSPORT_VERSION 3
-#else
-#define STP_TRANSPORT_VERSION 2
-#endif
-#else
-/* older kernels have no debugfs and older version of relayfs. */
-#define STP_TRANSPORT_VERSION 1
-#endif
 
 #if defined(STAPCONF_UDELAY_SIMPLE_EXPORTED)
 #undef udelay
@@ -89,9 +100,11 @@ static void _stp_exit(void);
 
 #ifdef STAPCONF_HLIST_4ARGS
 #define stap_hlist_for_each_entry(a,b,c,d) hlist_for_each_entry(a,b,c,d)
+#define stap_hlist_for_each_entry_rcu(a,b,c,d) hlist_for_each_entry_rcu(a,b,c,d)
 #define stap_hlist_for_each_entry_safe(a,b,c,d,e) hlist_for_each_entry_safe(a,b,c,d,e)
 #else
 #define stap_hlist_for_each_entry(a,b,c,d) (void) b; hlist_for_each_entry(a,c,d)
+#define stap_hlist_for_each_entry_rcu(a,b,c,d) (void) b; hlist_for_each_entry_rcu(a,c,d)
 #define stap_hlist_for_each_entry_safe(a,b,c,d,e) (void) b; hlist_for_each_entry_safe(a,c,d,e)
 #endif
 
