@@ -1,6 +1,6 @@
 /*
  * syscall defines and inlines
- * Copyright (C) 2008-2009 Red Hat Inc.
+ * Copyright (C) 2008-2019 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -185,6 +185,16 @@ _stp_syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
 #define _stp_syscall_get_nr syscall_get_nr
 #endif
 
+/* The syscall_get_arguments function arguments changed due to
+ * Linux git commit 32d9258662.  Removed the unused arguments
+ * to match the expect arguments for syscall_get_arguments when needed.
+ */
+#ifdef STAPCONF_SYSCALL_GET_ARGS_3ARGS
+#define _stp_syscall_get_arguments(t,r,i,n,a) syscall_get_arguments(t,r,a)
+#else
+#define _stp_syscall_get_arguments(t,r,i,n,a) syscall_get_arguments(t,r,i,n,a)
+#endif
+
 #else  /* !STAPCONF_ASM_SYSCALL_H */
 
 /* If the system doesn't have asm/syscall.h, use our defines. */
@@ -317,9 +327,10 @@ syscall_get_return_value(struct task_struct *task, struct pt_regs *regs)
 }
 
 #endif
+
 #if defined(__i386__) || defined(__x86_64__)
 static inline void
-syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
+_stp_syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 		      unsigned int i, unsigned int n, unsigned long *args)
 {
 	if (i + n > 6) {
@@ -340,18 +351,23 @@ syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 		case 0:
 			if (!n--) break;
 			*args++ = regs->bx;
+			/* fallthrough */
 		case 1:
 			if (!n--) break;
 			*args++ = regs->cx;
+			/* fallthrough */
 		case 2:
 			if (!n--) break;
 			*args++ = regs->dx;
+			/* fallthrough */
 		case 3:
 			if (!n--) break;
 			*args++ = regs->si;
+			/* fallthrough */
 		case 4:
 			if (!n--) break;
 			*args++ = regs->di;
+			/* fallthrough */
 		case 5:
 			if (!n--) break;
 			*args++ = regs->bp;
@@ -359,18 +375,23 @@ syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 		case 0:
 			if (!n--) break;
 			*args++ = regs->rbx;
+			/* fallthrough */
 		case 1:
 			if (!n--) break;
 			*args++ = regs->rcx;
+			/* fallthrough */
 		case 2:
 			if (!n--) break;
 			*args++ = regs->rdx;
+			/* fallthrough */
 		case 3:
 			if (!n--) break;
 			*args++ = regs->rsi;
+			/* fallthrough */
 		case 4:
 			if (!n--) break;
 			*args++ = regs->rdi;
+			/* fallthrough */
 		case 5:
 			if (!n--) break;
 			*args++ = regs->rbp;
@@ -384,18 +405,23 @@ syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 	case 0:
 		if (!n--) break;
 		*args++ = regs->di;
+		/* fallthrough */
 	case 1:
 		if (!n--) break;
 		*args++ = regs->si;
+		/* fallthrough */
 	case 2:
 		if (!n--) break;
 		*args++ = regs->dx;
+		/* fallthrough */
 	case 3:
 		if (!n--) break;
 		*args++ = regs->r10;
+		/* fallthrough */
 	case 4:
 		if (!n--) break;
 		*args++ = regs->r8;
+		/* fallthrough */
 	case 5:
 		if (!n--) break;
 		*args++ = regs->r9;
@@ -403,18 +429,23 @@ syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 	case 0:
 		if (!n--) break;
 		*args++ = regs->rdi;
+		/* fallthrough */
 	case 1:
 		if (!n--) break;
 		*args++ = regs->rsi;
+		/* fallthrough */
 	case 2:
 		if (!n--) break;
 		*args++ = regs->rdx;
+		/* fallthrough */
 	case 3:
 		if (!n--) break;
 		*args++ = regs->r10;
+		/* fallthrough */
 	case 4:
 		if (!n--) break;
 		*args++ = regs->r8;
+		/* fallthrough */
 	case 5:
 		if (!n--) break;
 		*args++ = regs->r9;
@@ -427,7 +458,7 @@ syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 
 #if defined(__powerpc__)
 static inline void
-syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
+_stp_syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 		      unsigned int i, unsigned int n, unsigned long *args)
 {
 	if (i + n > 6) {
@@ -531,7 +562,7 @@ void ia64_syscall_get_set_arguments(struct task_struct *task,
 	}
 }
 
-static inline void syscall_get_arguments(struct task_struct *task,
+static inline void _stp_syscall_get_arguments(struct task_struct *task,
 					 struct pt_regs *regs,
 					 unsigned int i, unsigned int n,
 					 unsigned long *args)
@@ -544,23 +575,30 @@ static inline void syscall_get_arguments(struct task_struct *task,
 		case 6:
 			if (!n--) break;
 			*args++ = regs->r13;
+			/* fallthrough */
 		case 5:
 			if (!n--) break;
 			*args++ = regs->r15;
+			/* fallthrough */
 		case 4:
 			if (!n--) break;
 			*args++ = regs->r14;
+			/* fallthrough */
 		case 3:
 			if (!n--) break;
 			*args++ = regs->r10;
+			/* fallthrough */
 		case 2:
 			if (!n--) break;
 			*args++ = regs->r9;
+			/* fallthrough */
 		case 1:
 			if (!n--) break;
 			*args++ = regs->r11;
+			/* fallthrough */
 		case 0:
 			if (!n--) break;
+			/* fallthrough */
 		default:
 			BUG();
 			break;
@@ -575,7 +613,7 @@ static inline void syscall_get_arguments(struct task_struct *task,
 
 #if defined(__s390__) || defined(__s390x__)
 static inline void
-syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
+_stp_syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 		      unsigned int i, unsigned int n, unsigned long *args)
 {
 	unsigned long mask = -1UL;
@@ -592,18 +630,23 @@ syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 	case 0:
 		if (!n--) break;
 		*args++ = regs->orig_gpr2 & mask;
+		/* fallthrough */
 	case 1:
 		if (!n--) break;
 		*args++ = regs->gprs[3] & mask;
+		/* fallthrough */
 	case 2:
 		if (!n--) break;
 		*args++ = regs->gprs[4] & mask;
+		/* fallthrough */
 	case 3:
 		if (!n--) break;
 		*args++ = regs->gprs[5] & mask;
+		/* fallthrough */
 	case 4:
 		if (!n--) break;
 		*args++ = regs->gprs[6] & mask;
+		/* fallthrough */
 	case 5:
 		if (!n--) break;
 		*args++ = regs->args[0] & mask;
@@ -613,7 +656,7 @@ syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 
 #if defined(__arm__)
 static inline void
-syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
+_stp_syscall_get_arguments(struct task_struct *task, struct pt_regs *regs,
 		      unsigned int i, unsigned int n, unsigned long *args)
 {
 	if (i + n > 6) {
