@@ -16,10 +16,14 @@
 
 #include "transport.h"
 #include "control.h"
+#include "../sym.h"
 #include <linux/debugfs.h>
 #include <linux/namei.h>
 #include <linux/delay.h>
 #include <linux/mutex.h>
+#ifdef CONFIG_SECURITY_LOCKDOWN_LSM
+#include <linux/security.h>
+#endif
 #include "../uidgid_compatibility.h"
 
 static int _stp_exit_flag = 0;
@@ -45,32 +49,25 @@ static int _stp_probes_started = 0;
  * transport state flag is atomic. */
 static atomic_t _stp_transport_state = ATOMIC_INIT(_STP_TS_UNINITIALIZED);
 
+static inline int _stp_trylock_inode(struct inode *inode);
+static inline void _stp_lock_inode(struct inode *inode);
+static inline void _stp_unlock_inode(struct inode *inode);
+
 #ifndef STP_CTL_TIMER_INTERVAL
 /* ctl timer interval in jiffies (default 20 ms) */
 #define STP_CTL_TIMER_INTERVAL		((HZ+49)/50)
 #endif
 
-
-// For now, disable transport version 3 (unless STP_USE_RING_BUFFER is
-// defined).
-#if STP_TRANSPORT_VERSION == 3 && !defined(STP_USE_RING_BUFFER)
-#undef STP_TRANSPORT_VERSION
-#define STP_TRANSPORT_VERSION 2
-#endif
+/* Defines the number of buffers allocated in control.c (which #includes
+   this file) for the _stp_pool_q.  This is the number of .cmd messages
+   the module can store before they have to be read by stapio.
+   40 is somewhat arbitrary, 8 pre-allocated messages, 32 dynamic.  */
+#define STP_DEFAULT_BUFFERS 256
 
 #include "control.h"
-#if STP_TRANSPORT_VERSION == 1
-#include "relayfs.c"
-#elif STP_TRANSPORT_VERSION == 2
 #include "relay_v2.c"
 #include "debugfs.c"
-#elif STP_TRANSPORT_VERSION == 3
-/* TODO: Ring buffer mechanism is deprecated and could be removed. */
-#include "ring_buffer.c"
-#include "debugfs.c"
-#else
-#error "Unknown STP_TRANSPORT_VERSION"
-#endif
+#include "procfs.c"
 #include "control.c"
 
 static unsigned _stp_nsubbufs = 8;
@@ -100,17 +97,108 @@ static struct notifier_block _stp_module_notifier_nb = {
         .priority = 0
 };
 
-#if STP_TRANSPORT_VERSION == 2
 static int _stp_module_panic_notifier (struct notifier_block * nb,
                                  unsigned long val, void *data);
 static struct notifier_block _stp_module_panic_notifier_nb = {
         .notifier_call = _stp_module_panic_notifier,
         .priority = INT_MAX
 };
-#endif
 
 static struct timer_list _stp_ctl_work_timer;
 
+
+
+// ------------------------------------------------------------------------
+
+// Dispatching functions to choose between procfs and debugfs variants
+// of the transport.  PR26665
+
+static int _stp_transport_fs_init(const char *module_name)
+{
+        // BTW: testing the other !FOO_p first is to protect against repeated
+        // invocations of this function with security_locked_down() changing
+#ifdef STAPCONF_LOCKDOWN_DEBUGFS
+        if (!debugfs_p && security_locked_down (LOCKDOWN_DEBUGFS)) {
+                procfs_p = 1;
+		dbug_trans(1, "choosing procfs_p=1\n");
+        }
+#endif
+        if (!procfs_p) {
+                debugfs_p = 1;
+		dbug_trans(1, "choosing debugfs_p=1\n");
+        }
+
+#ifdef STAP_TRANS_PROCFS
+        procfs_p = 1;
+        debugfs_p = 0;
+        dbug_trans(1, "forcing procfs_p=1\n");
+#endif
+#ifdef STAP_TRANS_DEBUGFS
+        procfs_p = 0;
+        debugfs_p = 1;
+        dbug_trans(1, "forcing debugfs_p=1\n");        
+#endif        
+        
+        if (debugfs_p)
+                return _stp_debugfs_transport_fs_init(module_name);        
+        if (procfs_p)
+                return _stp_procfs_transport_fs_init(module_name);
+        return -ENOSYS;
+}
+
+static void _stp_transport_fs_close(void)
+{
+        if (debugfs_p)
+                _stp_debugfs_transport_fs_close();
+        if (procfs_p)
+                _stp_procfs_transport_fs_close();
+}
+
+
+static int _stp_ctl_write_fs(int type, void *data, unsigned len)
+{
+        if (procfs_p)
+                return _stp_procfs_ctl_write_fs (type, data, len);
+        if (debugfs_p)
+                return _stp_debugfs_ctl_write_fs (type, data, len);
+        return -ENOSYS;
+}
+
+
+static int _stp_register_ctl_channel_fs(void)
+{
+        if (debugfs_p)
+                return _stp_debugfs_register_ctl_channel_fs();
+        if (procfs_p)
+                return _stp_procfs_register_ctl_channel_fs();
+
+        return -ENOSYS;
+}
+
+
+static void _stp_unregister_ctl_channel_fs(void)
+{
+        if (procfs_p)
+                _stp_procfs_unregister_ctl_channel_fs();
+        if (debugfs_p)
+                _stp_debugfs_unregister_ctl_channel_fs();
+}
+
+
+
+static struct dentry *_stp_get_module_dir(void)
+{
+        if (procfs_p)
+                return _stp_procfs_get_module_dir();
+        if (debugfs_p)
+                return _stp_debugfs_get_module_dir();
+        return NULL;
+}
+
+
+
+// ------------------------------------------------------------------------
+
 /*
  *	_stp_handle_start - handle STP_START
  */
@@ -204,9 +292,7 @@ static void _stp_handle_start(struct _stp_msg_start *st)
 		_stp_ctl_send_notify(STP_START, st, sizeof(*st));
 
 		/* Register the panic notifier. */
-#if STP_TRANSPORT_VERSION == 2
 		atomic_notifier_chain_register(&panic_notifier_list, &_stp_module_panic_notifier_nb);
-#endif
 
 		/* If we're here, we know that _stp_transport_state
 		 * is _STP_TS_START_CALLED. Go ahead and set it to
@@ -215,6 +301,77 @@ static void _stp_handle_start(struct _stp_msg_start *st)
 	}
 }
 
+#if defined(CONFIG_KALLSYMS) && !defined(STAPCONF_KALLSYMS_LOOKUP_NAME_EXPORTED)
+unsigned _stp_need_kallsyms_stext;
+#endif
+
+// PR26074: Most kallsyms_lookup_name() calls are done in appropriate
+// subsystem init() functions, but a handful are done at module load
+// time:
+static int _stp_handle_kallsyms_lookups(void)
+{
+#if !defined(STAPCONF_SET_FS)
+  /* PR26811, missing copy_to_kernel_nofault symbol-export */
+  kallsyms_copy_to_kernel_nofault = (void*) kallsyms_lookup_name ("copy_to_kernel_nofault");
+  /* Not finding this symbol is non-fatal. Kernel writes will fault safely: */
+  if (kallsyms_copy_to_kernel_nofault == NULL) {
+    ;
+  }
+#endif
+  /* PR13489, missing inode-uprobes symbol-export workaround */
+#if !defined(STAPCONF_TASK_USER_REGSET_VIEW_EXPORTED) && !defined(STAPCONF_UTRACE_REGSET) /* RHEL5 era utrace */
+        kallsyms_task_user_regset_view = (void*) kallsyms_lookup_name ("task_user_regset_view");
+        /* There exist interesting kernel versions without task_user_regset_view(), like ARM before 3.0.
+           For these kernels, uprobes etc. are out of the question, but plain kernel stap works fine.
+           All we have to accomplish is have the loc2c runtime code compile.  For that, it's enough
+           to leave this pointer zero. */
+        if (kallsyms_task_user_regset_view == NULL) {
+                ;
+        }
+#endif
+#if defined(CONFIG_UPROBES) // i.e., kernel-embedded uprobes
+#if !defined(STAPCONF_UPROBE_REGISTER_EXPORTED)
+        kallsyms_uprobe_register = (void*) kallsyms_lookup_name ("uprobe_register");
+        if (kallsyms_uprobe_register == NULL) {
+                kallsyms_uprobe_register = (void*) kallsyms_lookup_name ("register_uprobe");
+        }
+        if (kallsyms_uprobe_register == NULL) {
+                _stp_error("Can't resolve uprobe_register!");
+                goto err0;
+        }
+#endif
+#if !defined(STAPCONF_UPROBE_UNREGISTER_EXPORTED)
+        kallsyms_uprobe_unregister = (void*) kallsyms_lookup_name ("uprobe_unregister");
+        if (kallsyms_uprobe_unregister == NULL) {
+                kallsyms_uprobe_unregister = (void*) kallsyms_lookup_name ("unregister_uprobe");
+        }
+        if (kallsyms_uprobe_unregister == NULL) {
+                _stp_error("Can't resolve uprobe_unregister!");
+                goto err0;
+        }
+#endif
+#if !defined(STAPCONF_UPROBE_GET_SWBP_ADDR_EXPORTED)
+        kallsyms_uprobe_get_swbp_addr = (void*) kallsyms_lookup_name ("uprobe_get_swbp_addr");
+        if (kallsyms_uprobe_get_swbp_addr == NULL) {
+                _stp_error("Can't resolve uprobe_get_swbp_addr!");
+                goto err0;
+        }
+#endif
+#endif
+#if defined(CONFIG_KALLSYMS) && !defined(STAPCONF_KALLSYMS_LOOKUP_NAME_EXPORTED)
+        {
+                uint64_t address;
+                if (_stp_need_kallsyms_stext) {
+                        address = kallsyms_lookup_name("_stext");
+                        _stp_set_stext(address);
+                        _stp_need_kallsyms_stext = 0;
+                }
+        }
+#endif
+        return 0;
+err0:
+        return -1;
+}
 
 // _stp_cleanup_and_exit: handle STP_EXIT and cleanup_module
 //
@@ -270,9 +427,7 @@ static void _stp_cleanup_and_exit(int send_exit)
 		dbug_trans(1, "done with ctl_send STP_EXIT\n");
 
 		/* Unregister the panic notifier. */
-#if STP_TRANSPORT_VERSION == 2
 		atomic_notifier_chain_unregister(&panic_notifier_list, &_stp_module_panic_notifier_nb);
-#endif
 	}
 }
 
@@ -406,45 +561,30 @@ static int _stp_transport_init(void)
 #endif
 #endif
 
-/* PR13489, missing inode-uprobes symbol-export workaround */
-#if !defined(STAPCONF_TASK_USER_REGSET_VIEW_EXPORTED) && !defined(STAPCONF_UTRACE_REGSET) /* RHEL5 era utrace */
-        kallsyms_task_user_regset_view = (void*) kallsyms_lookup_name ("task_user_regset_view");
-        /* There exist interesting kernel versions without task_user_regset_view(), like ARM before 3.0.
-           For these kernels, uprobes etc. are out of the question, but plain kernel stap works fine.
-           All we have to accomplish is have the loc2c runtime code compile.  For that, it's enough
-           to leave this pointer zero. */
-        if (kallsyms_task_user_regset_view == NULL) {
-                ;
-        }
+        /* PR26074: kallsyms_lookup_name() invocations are now performed later.
+           Just in case, we should init some of the variables to NULL. */
+#if !defined(STAPCONF_TASK_USER_REGSET_VIEW_EXPORTED)
+        kallsyms_task_user_regset_view = NULL;
 #endif
 #if defined(CONFIG_UPROBES) // i.e., kernel-embedded uprobes
 #if !defined(STAPCONF_UPROBE_REGISTER_EXPORTED)
-        kallsyms_uprobe_register = (void*) kallsyms_lookup_name ("uprobe_register");
-        if (kallsyms_uprobe_register == NULL) {
-		kallsyms_uprobe_register = (void*) kallsyms_lookup_name ("register_uprobe");
-        }
-        if (kallsyms_uprobe_register == NULL) {
-                printk(KERN_ERR "%s can't resolve uprobe_register!", THIS_MODULE->name);
-                goto err0;
-        }
+        kallsyms_uprobe_register = NULL;
 #endif
 #if !defined(STAPCONF_UPROBE_UNREGISTER_EXPORTED)
-        kallsyms_uprobe_unregister = (void*) kallsyms_lookup_name ("uprobe_unregister");
-        if (kallsyms_uprobe_unregister == NULL) {
-		kallsyms_uprobe_unregister = (void*) kallsyms_lookup_name ("unregister_uprobe");
-        }
-        if (kallsyms_uprobe_unregister == NULL) {
-                printk(KERN_ERR "%s can't resolve uprobe_unregister!", THIS_MODULE->name);
-                goto err0;
-        }
+        kallsyms_uprobe_unregister = NULL;
 #endif
 #if !defined(STAPCONF_UPROBE_GET_SWBP_ADDR_EXPORTED)
-        kallsyms_uprobe_get_swbp_addr = (void*) kallsyms_lookup_name ("uprobe_get_swbp_addr");
-        if (kallsyms_uprobe_get_swbp_addr == NULL) {
-                printk(KERN_ERR "%s can't resolve uprobe_get_swbp_addr!", THIS_MODULE->name);
-                goto err0;
-        }
+        kallsyms_uprobe_get_swbp_addr = NULL;
 #endif
+#endif
+#if !defined(STAPCONF_KALLSYMS_LOOKUP_NAME_EXPORTED)
+        _stp_kallsyms_lookup_name = NULL;
+#endif
+#if defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL) && !defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL_EXPORTED)
+        _stp_kallsyms_on_each_symbol = NULL;
+#endif
+#if defined(CONFIG_KALLSYMS) && !defined(STAPCONF_KALLSYMS_LOOKUP_NAME_EXPORTED)
+        _stp_need_kallsyms_stext = 0;
 #endif
 
 	if (_stp_bufsize) {
@@ -497,6 +637,23 @@ err0:
 	return -1;
 }
 
+/* returns 1 when the lock is successfully acquired, 0 otherwise. */
+static inline int _stp_trylock_inode(struct inode *inode)
+{
+#ifdef STAPCONF_INODE_RWSEM
+	return inode_trylock(inode);
+#else
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+	return mutex_trylock(&inode->i_mutex);
+#else
+	/* NB down_trylock() uses a different convention where 0 means
+	 * the lock is successfully acquired.
+	 */
+	return !down_trylock(&inode->i_sem);
+#endif
+#endif
+}
+
 static inline void _stp_lock_inode(struct inode *inode)
 {
 #ifdef STAPCONF_INODE_RWSEM
@@ -524,209 +681,6 @@ static inline void _stp_unlock_inode(struct inode *inode)
 #endif
 }
 
-static struct dentry *_stp_lockfile = NULL;
-
-static int _stp_lock_transport_dir(void)
-{
-	int numtries = 0;
-
-#if STP_TRANSPORT_VERSION == 1
-	while ((_stp_lockfile = relayfs_create_dir("systemtap_lock", NULL)) == NULL) {
-#else
-	while ((_stp_lockfile = debugfs_create_dir("systemtap_lock", NULL)) == NULL) {
-#endif
-		if (numtries++ >= 50)
-			return 0;
-		msleep(50);
-	}
-	return 1;
-}
-
-static void _stp_unlock_transport_dir(void)
-{
-	if (_stp_lockfile) {
-#if STP_TRANSPORT_VERSION == 1
-		relayfs_remove_dir(_stp_lockfile);
-#else
-		debugfs_remove(_stp_lockfile);
-#endif
-		_stp_lockfile = NULL;
-	}
-}
-
-static struct dentry *__stp_root_dir = NULL;
-
-/* _stp_get_root_dir() - creates root directory or returns
- * a pointer to it if it already exists.
- *
- * The caller *must* lock the transport directory.
- */
-
-static struct dentry *_stp_get_root_dir(void)
-{
-	struct file_system_type *fs;
-	struct super_block *sb;
-	const char *name = "systemtap";
-
-	if (__stp_root_dir != NULL) {
-		return __stp_root_dir;
-	}
-
-#if STP_TRANSPORT_VERSION == 1
-	fs = get_fs_type("relayfs");
-	if (!fs) {
-		errk("Couldn't find relayfs filesystem.\n");
-		return NULL;
-	}
-#else
-	fs = get_fs_type("debugfs");
-	if (!fs) {
-		errk("Couldn't find debugfs filesystem.\n");
-		return NULL;
-	}
-#endif
-
-#if STP_TRANSPORT_VERSION == 1
-	__stp_root_dir = relayfs_create_dir(name, NULL);
-#else
-	__stp_root_dir = debugfs_create_dir(name, NULL);
-#endif
-	if (__stp_root_dir == ERR_PTR(-EEXIST)) /* some kernels signal duplication this way */
-	  __stp_root_dir = NULL;
-	if (!__stp_root_dir) {
-		/* Couldn't create it because it is already there, so
-		 * find it. */
-#ifdef STAPCONF_FS_SUPERS_HLIST
-		sb = hlist_entry(fs->fs_supers.first, struct super_block,
-	 			 s_instances);
-#else
-		sb = list_entry(fs->fs_supers.next, struct super_block,
-				s_instances);
-#endif
-		_stp_lock_inode(sb->s_root->d_inode);
-		__stp_root_dir = lookup_one_len(name, sb->s_root,
-						strlen(name));
-		_stp_unlock_inode(sb->s_root->d_inode);
-		if (!IS_ERR(__stp_root_dir))
-			dput(__stp_root_dir);
-		else {
-			__stp_root_dir = NULL;
-			errk("Could not create or find transport directory.\n");
-		}
-	}
-	else if (IS_ERR(__stp_root_dir)) {
-	    __stp_root_dir = NULL;
-	    errk("Could not create root directory \"%s\", error %ld\n", name,
-		 -PTR_ERR(__stp_root_dir));
-	}
-
-	return __stp_root_dir;
-}
-
-/* _stp_remove_root_dir() - removes root directory (if empty)
- *
- * The caller *must* lock the transport directory.
- */
-
-static void _stp_remove_root_dir(void)
-{
-	if (__stp_root_dir) {
-		if (simple_empty(__stp_root_dir)) {
-#if STP_TRANSPORT_VERSION == 1
-			relayfs_remove_dir(__stp_root_dir);
-#else
-			debugfs_remove(__stp_root_dir);
-#endif
-		}
-		__stp_root_dir = NULL;
-	}
-}
-
-static struct dentry *__stp_module_dir = NULL;
-
-static struct dentry *_stp_get_module_dir(void)
-{
-	return __stp_module_dir;
-}
-
-static int _stp_transport_fs_init(const char *module_name)
-{
-	struct dentry *root_dir;
-    
-	dbug_trans(1, "entry\n");
-	if (module_name == NULL)
-		return -1;
-
-	if (!_stp_lock_transport_dir()) {
-		errk("Couldn't lock transport directory.\n");
-		return -1;
-	}
-
-	root_dir = _stp_get_root_dir();
-	if (root_dir == NULL) {
-		_stp_unlock_transport_dir();
-		return -1;
-	}
-
-#if STP_TRANSPORT_VERSION == 1
-        __stp_module_dir = relayfs_create_dir(module_name, root_dir);
-#else
-        __stp_module_dir = debugfs_create_dir(module_name, root_dir);
-#endif
-        if (!__stp_module_dir) {
-		errk("Could not create module directory \"%s\"\n",
-		     module_name);
-		_stp_remove_root_dir();
-		_stp_unlock_transport_dir();
-		return -1;
-	}
-	else if (IS_ERR(__stp_module_dir)) {
-		errk("Could not create module directory \"%s\", error %ld\n",
-		     module_name, -PTR_ERR(__stp_module_dir));
-		_stp_remove_root_dir();
-		_stp_unlock_transport_dir();
-		return -1;
-	}
-
-	if (_stp_transport_data_fs_init() != 0) {
-#if STP_TRANSPORT_VERSION == 1
-		relayfs_remove_dir(__stp_module_dir);
-#else
-		debugfs_remove(__stp_module_dir);
-#endif
-		__stp_module_dir = NULL;
-		_stp_remove_root_dir();
-		_stp_unlock_transport_dir();
-		return -1;
-	}
-	_stp_unlock_transport_dir();
-	dbug_trans(1, "returning 0\n");
-	return 0;
-}
-
-static void _stp_transport_fs_close(void)
-{
-	dbug_trans(1, "stp_transport_fs_close\n");
-
-	_stp_transport_data_fs_close();
-
-	if (__stp_module_dir) {
-		if (!_stp_lock_transport_dir()) {
-			errk("Couldn't lock transport directory.\n");
-			return;
-		}
-
-#if STP_TRANSPORT_VERSION == 1
-		relayfs_remove_dir(__stp_module_dir);
-#else
-		debugfs_remove(__stp_module_dir);
-#endif
-		__stp_module_dir = NULL;
-
-		_stp_remove_root_dir();
-		_stp_unlock_transport_dir();
-	}
-}
 
 
 /* NB: Accessed from tzinfo.stp tapset */
