@@ -62,7 +62,11 @@ enum
 	    into the module).  */
 	STP_READY,
 	/** Send by staprun at startup to notify module of where the kernel
-	    (_stext) and all other modules are loaded.  */
+	    (_stext) and all other modules are loaded.
+
+            PR26074: Also used to pass addresses of some unexported
+            kernel functions, most notably kallsyms_lookup_name (which
+            is used to retrieve yet other unexported functions). */
         STP_RELOCATION,
 	/** Never used.  deprecated STP_TRANSPORT_VERSION == 1 **/
 	STP_BUF_INFO,
@@ -145,22 +149,6 @@ struct _stp_msg_ns_pid
   pid_t target;
 };
 
-#if STP_TRANSPORT_VERSION == 1
-/**** for compatibility with old relayfs ****/
-struct _stp_buf_info
-{
-        int32_t cpu;
-        uint32_t produced;
-        uint32_t consumed;
-        int32_t flushing;
-};
-struct _stp_consumed_info
-{
-        int32_t cpu;
-        uint32_t consumed;
-};
-#endif
-
 /* Unwind data. stapio->module */
 struct _stp_msg_relocation
 {
