@@ -160,7 +160,7 @@ static unsigned long read_ptr_sect(const u8 **pLoc, const void *end,
 #else
 		BUILD_BUG_ON(sizeof(u32) != sizeof(value));
 #endif
-		/* fallthrough, see above. */
+		/* fallthrough */
 	case DW_EH_PE_absptr:
 		if (compat_task)
 		{
@@ -202,7 +202,7 @@ static unsigned long read_ptr_sect(const u8 **pLoc, const void *end,
 	if (ptrType & DW_EH_PE_indirect
 	    && _stp_deref_nofault(value, sizeof(value),
 				  (unsigned long *)value, 
-				  (user ? USER_DS : KERNEL_DS)))
+				  (user ? STP_USER_DS : STP_KERNEL_DS)))
 		return 0;
 	*pLoc = ptr.p8;
 
