@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2018 Red Hat Inc.
+// Copyright (C) 2005-2019 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -122,6 +122,9 @@ struct parse_error: public std::runtime_error
     }
 };
 
+
+class symresolution_info;
+
 struct systemtap_session
 {
 private:
@@ -255,6 +258,7 @@ public:
   bool color_errors;
   bool interactive_mode;
   bool run_example;
+  bool no_global_var_display;
   bool pass_1a_complete;
 
   enum { color_never, color_auto, color_always } color_mode;
@@ -262,6 +266,7 @@ public:
 
   enum { kernel_runtime, dyninst_runtime, bpf_runtime } runtime_mode;
   bool runtime_usermode_p() const { return runtime_mode == dyninst_runtime; }
+  bool use_bpf_raw_tracepoint;
 
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
@@ -335,7 +340,8 @@ public:
   // temporary directory for module builds etc.
   // hazardous - it is "rm -rf"'d at exit
   std::string tmpdir;
-  std::string translated_source; // C source code
+  std::string translated_source; // main C source code
+  std::string symbols_source; // C source code for stap_symbols.c
 
   match_node* pattern_root;
   void register_library_aliases();
@@ -368,6 +374,7 @@ public:
   std::vector<vardecl*> unused_globals;
   std::vector<derived_probe*> unused_probes; // see also *_probes groups below
   std::vector<functiondecl*> unused_functions;
+  std::set<derived_probe*> empty_probes;
 
   // resolved/compiled regular expressions for the run
   std::map<std::string, stapdfa*> dfas;
@@ -447,6 +454,7 @@ public:
   unsigned suppress_costly_diagnostics; /* set during processing of optional probes */
 
   const token* last_token;
+
   void print_token (std::ostream& o, const token* tok);
   void print_error (const semantic_error& e);
   std::string build_error_msg (const semantic_error& e);
@@ -463,6 +471,9 @@ public:
   void printscript(std::ostream& o);
   void report_suppression();
 
+  // PR25841, for early probe-derivation-time symbol resolution
+  symresolution_info* symbol_resolver; // may be NULL
+  
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
 
