@@ -9,11 +9,10 @@
 #ifndef MUTATOR_H
 #define MUTATOR_H
 
+#include <memory>
 #include <string>
 #include <vector>
 
-#include <boost/shared_ptr.hpp>
-
 #include <BPatch.h>
 #include <BPatch_module.h>
 #include <BPatch_process.h>
@@ -38,11 +37,12 @@ class mutator {
     std::string module_shmem; // the global name of this module's shared memory
     std::vector<dynprobe_target> targets; // the probe targets in the module
 
-    std::vector<boost::shared_ptr<mutatee> > mutatees; // all attached target processes
-    boost::shared_ptr<mutatee> target_mutatee; // the main target process we created or attached
+    std::vector<std::shared_ptr<mutatee> > mutatees; // all attached target processes
+    std::shared_ptr<mutatee> target_mutatee; // the main target process we created or attached
     bool p_target_created; // we only kill and wait on the target we created
+    bool p_target_error; // indicates whether the target exited non-zero;
 
-    unsigned signal_count; // how many exit signals we've received
+    sigset_t signals_received; // record all signals we've caught
 
     // disable implicit constructors by not implementing these
     mutator (const mutator& other);
@@ -51,6 +51,9 @@ class mutator {
     // Initialize the module global variables
     bool init_modoptions();
 
+    // Initialize the session attributes
+    void init_session_attributes();
+
     // Initialize the module session
     bool run_module_init();
 
@@ -64,10 +67,10 @@ class mutator {
     bool matching_probes_exist(uint64_t flag);
 
     // Find a mutatee which matches the given process, else return NULL
-    boost::shared_ptr<mutatee> find_mutatee(BPatch_process* process);
+    std::shared_ptr<mutatee> find_mutatee(BPatch_process* process);
 
     // Stashed utrace probe enter function pointer.
-    typeof(&enter_dyninst_utrace_probe) utrace_enter_fn;
+    decltype(&enter_dyninst_utrace_probe) utrace_enter_fn;
   public:
 
     mutator (const std::string& module_name,
@@ -87,10 +90,13 @@ class mutator {
     // Start the actual systemtap session!
     bool run ();
 
+    // Get the final exit status of this mutator
+    int exit_status();
+
     // Callback to respond to dynamically loaded libraries.
     // Check if it matches our targets, and instrument accordingly.
     void dynamic_library_callback(BPatch_thread *thread,
-                                  BPatch_module *module,
+                                  BPatch_object *object,
                                   bool load);
 
     // Callback to respond to post fork events.  Check if it matches
