@@ -7,7 +7,7 @@
  * Public License (GPL); either version 2, or (at your option) any
  * later version.
  *
- * Copyright (C) 2005-2008 Red Hat Inc.
+ * Copyright (C) 2005-2016 Red Hat Inc.
  */
 #define _FILE_OFFSET_BITS 64
 
@@ -142,9 +142,6 @@ extern char *__name__;
 /* Grabbed from linux/module.h kernel include. */
 #define MODULE_NAME_LEN (64 - sizeof(unsigned long))
 
-/* We define this so we are compatible with old transport, but we
- * don't have to use it. */
-#define STP_TRANSPORT_VERSION 1
 #include "../runtime/transport/transport_msgs.h"
 
 #define RELAYFS_MAGIC	0xF0B4A981
@@ -164,6 +161,7 @@ int init_ctl_channel(const char *name, int verb);
 void close_ctl_channel(void);
 int init_relayfs(void);
 void close_relayfs(void);
+void kill_relayfs(void);
 int init_oldrelayfs(void);
 void close_oldrelayfs(int);
 int write_realtime_data(void *data, ssize_t nb);
@@ -173,6 +171,8 @@ int make_outfile_name(char *buf, int max, int fnum, int cpu,
 int init_backlog(int cpu);
 void write_backlog(int cpu, int fnum, time_t t);
 time_t read_backlog(int cpu, int fnum);
+void read_stdin_setup(void);
+void read_stdin_cleanup(void);
 /* staprun_funcs.c */
 void setup_staprun_signals(void);
 const char *moderror(int err);
@@ -220,6 +220,21 @@ void usage(char *prog, int rc);
 void parse_modpath(const char *);
 void setup_signals(void);
 int set_clexec(int fd);
+int open_cloexec(const char *pathname, int flags, mode_t mode);
+#ifdef HAVE_OPENAT
+int openat_cloexec(int dirfd, const char *pathname, int flags, mode_t mode);
+#endif
+int pipe_cloexec(int pipefd[2]);
+void closefrom(int lowfd);
+
+/* monitor.c function */
+void monitor_winch(int signum);
+void monitor_setup(void);
+void monitor_cleanup(void);
+void monitor_render(void);
+void monitor_input(void);
+void monitor_exited(void);
+void monitor_remember_output_line(const char* buf, const size_t bytes);
 
 /*
  * variables
@@ -228,6 +243,8 @@ extern int control_channel;
 extern int ncpus;
 extern int initialized;
 extern int kernel_ptr_size;
+extern int monitor_pfd[2];
+extern int monitor_end;
 
 /* flags */
 extern int verbose;
@@ -242,6 +259,7 @@ extern int target_pid;
 extern char *target_cmd;
 extern int target_namespaces_pid;
 extern char *outfile_name;
+extern int read_stdin;
 extern int rename_mod;
 extern int attach_mod;
 extern int delete_mod;
@@ -255,6 +273,8 @@ extern int remote_id;
 extern const char *remote_uri;
 extern int relay_basedir_fd;
 extern int color_errors;
+extern int monitor;
+extern int monitor_interval;
 
 typedef enum {color_never, color_auto, color_always} color_modes;
 extern color_modes color_mode;
