@@ -1,5 +1,5 @@
 // parse tree functions
-// Copyright (C) 2005-2016 Red Hat Inc.
+// Copyright (C) 2005-2019 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -212,7 +212,7 @@ vardecl::compatible_arity (int a)
 
 
 functiondecl::functiondecl ():
-  body (0), synthetic (false), mangle_oldstyle (false), has_next(false),
+  body (0), synthetic (false), mangle_oldstyle (false), has_next(false), cloned_p(false),
   priority(1)
 {
 }
@@ -343,31 +343,67 @@ string atvar_op::sym_name ()
 }
 
 
+// Substring operations like interned_string.find() are repeated a
+// surprising number of times (with nested / widely used tapset emb-C
+// functions), and need to be efficient.  We memoize the search
+// results.
+bool memo_tagged_p (const interned_string& haystack, const string& needle)
+{
+  static map <pair<interned_string,string>,bool> string_find_memoized;
+  auto key = make_pair(haystack,needle);
+  
+  auto it = string_find_memoized.find(key);
+  if (it != string_find_memoized.end())
+    return it->second;
+
+  auto findres = haystack.find(needle);
+  bool res = (findres != interned_string::npos);
+  string_find_memoized.insert(make_pair(key,res));
+  
+  return res;
+}
+
+
+
 bool
 embedded_expr::tagged_p (const char *tag) const
 {
-  return code.find(tag) != interned_string::npos;
+  return memo_tagged_p (code, tag);
+}
+
+
+bool
+embedded_expr::tagged_p (const string &tag) const
+{
+  return memo_tagged_p (code, tag);
 }
 
 
 bool
 embedded_expr::tagged_p (const interned_string& tag) const
 {
-  return code.find(tag) != interned_string::npos;
+  return memo_tagged_p (code, tag);
 }
 
 
 bool
 embeddedcode::tagged_p (const char *tag) const
 {
-  return code.find(tag) != interned_string::npos;
+  return memo_tagged_p (code, tag);
+}
+
+
+bool
+embeddedcode::tagged_p (const string &tag) const
+{
+  return memo_tagged_p (code, tag);
 }
 
 
 bool
 embeddedcode::tagged_p (const interned_string& tag) const
 {
-  return code.find(tag) != interned_string::npos;
+  return memo_tagged_p (code, tag);
 }
 
 
@@ -520,7 +556,10 @@ void atvar_op::print (ostream& o) const
 {
   if (addressof)
     o << "&";
-  o << name << "(\"" << target_name << "\")";
+  o << name << "(\"" << target_name << "\"";
+  if (module.length() > 0)
+    o << ", " << lex_cast_qstring (module);
+  o << ')';
   for (unsigned i = 0; i < components.size(); ++i)
     o << components[i];
 }
@@ -556,6 +595,12 @@ void defined_op::print (ostream& o) const
 }
 
 
+void probewrite_op::print (ostream& o) const
+{
+  o << "@probewrite(" << name << ")";
+}
+
+
 void entry_op::print (ostream& o) const
 {
   o << "@entry(" << *operand << ")";
@@ -1209,6 +1254,13 @@ block::block (statement* car, statement* cdr)
   this->tok = car->tok;
 }
 
+block::block (statement* car, statement* cdr1, statement* cdr2)
+{
+  statements.push_back(car);
+  statements.push_back(cdr1);
+  statements.push_back(cdr2);
+  this->tok = car->tok;
+}
 
 
 void try_block::print (ostream& o) const
@@ -1301,13 +1353,19 @@ void null_statement::print (ostream& o) const
 
 void expr_statement::print (ostream& o) const
 {
-  o << *value;
+  if (value)
+    o << *value;
+  else
+    o << ";"; /* mid-elision-pass null-statement */
 }
 
 
 void return_statement::print (ostream& o) const
 {
-  o << "return " << *value;
+  if (value)
+    o << "return " << *value;
+  else
+    o << "return";
 }
 
 
@@ -1333,8 +1391,8 @@ void continue_statement::print (ostream& o) const
 
 void if_statement::print (ostream& o) const
 {
-  o << "if (" << *condition << ") "
-    << *thenblock << endl;
+  o << "if (" << *condition << ") ";
+  if (thenblock) o << *thenblock; else o << ";"; o << endl;
   if (elseblock)
     o << "else " << *elseblock << endl;
 }
@@ -1459,7 +1517,7 @@ string probe_point::str (bool print_extras) const
 
 
 probe_alias::probe_alias(std::vector<probe_point*> const & aliases):
-  probe (), alias_names (aliases), epilogue_style(false)
+  probe (), alias_names (aliases), epilogue_style(false), body2(0)
 {
 }
 
@@ -1761,6 +1819,12 @@ defined_op::visit (visitor* u)
 }
 
 
+void
+probewrite_op::visit (visitor* u)
+{
+  u->visit_probewrite_op(this);
+}
+
 void
 entry_op::visit (visitor* u)
 {
@@ -1958,7 +2022,8 @@ traversing_visitor::visit_foreach_loop (foreach_loop* s)
 void
 traversing_visitor::visit_return_statement (return_statement* s)
 {
-  s->value->visit (this);
+  if (s->value)
+    s->value->visit (this);
 }
 
 void
@@ -2140,6 +2205,14 @@ traversing_visitor::visit_defined_op (defined_op* e)
   e->operand->visit (this);
 }
 
+
+void
+traversing_visitor::visit_probewrite_op (probewrite_op* e)
+{
+  throw SEMANTIC_ERROR(_("unexpected @probewrite"), e->tok);
+}
+
+
 void
 traversing_visitor::visit_entry_op (entry_op* e)
 {
@@ -2453,6 +2526,82 @@ functioncall_traversing_visitor::note_recursive_functioncall (functioncall*)
 {
 }
 
+
+void
+symuse_collecting_visitor::visit_try_block(try_block* s)
+{
+  // Default to varuse visitor behaviour if the referent is set.
+  if (s->catch_error_var->referent)
+    varuse_collecting_visitor::visit_try_block(s);
+  else
+    {
+      if (s->try_block)
+        s->try_block->visit(this);
+      if (s->catch_block)
+        s->catch_block->visit(this);
+    }
+
+  // Add the name regardless of whether the referent is set or not.
+  if (s->catch_error_var)
+    written_names.insert(s->catch_error_var->name);
+}
+
+
+void
+symuse_collecting_visitor::visit_embeddedcode(embeddedcode* s)
+{
+  // If either of the referent vectors have a single element, then
+  // we can default to varuse visitor behaviour.
+  if (!s->read_referents.empty() || !s->write_referents.empty())
+    varuse_collecting_visitor::visit_embeddedcode(s);
+}
+
+
+void
+symuse_collecting_visitor::visit_embedded_expr(embedded_expr* e)
+{
+  // Similar to symuse_collecting_visitor::visit_embeddedcode.
+  if (!e->read_referents.empty() || !e->write_referents.empty())
+    varuse_collecting_visitor::visit_embedded_expr(e);
+}
+
+
+void
+symuse_collecting_visitor::visit_target_symbol(target_symbol* e)
+{
+  varuse_collecting_visitor::visit_target_symbol(e);
+
+  // We treat target symbols differently under the symuse visitor
+  // when compared to the varuse visitor. Target symbol names are
+  // added to the name sets that track reads and writes.
+  if (is_active_lvalue(e))
+    written_names.insert(e->name);
+  else
+    read_names.insert(e->name);
+}
+
+
+void
+symuse_collecting_visitor::visit_symbol(symbol* e)
+{
+  // Similar to the varuse visitor case. The symbol names are also tracked.
+  if (e->referent)
+    varuse_collecting_visitor::visit_symbol(e);
+
+  if (current_lvalue == e || current_lrvalue == e)
+    written_names.insert(e->name);
+
+  if (current_lvalue != e || current_lrvalue)
+    read_names.insert(e->name);
+}
+
+
+void
+symuse_collecting_visitor::visit_probewrite_op(probewrite_op*)
+{
+}
+
+
 void
 varuse_collecting_visitor::visit_if_statement (if_statement *s)
 {
@@ -2530,56 +2679,29 @@ varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
 {
   assert (current_function); // only they get embedded code
 
-  /* We need to lock globals that are accessed through embedded C code */
-  for (unsigned i = 0; i < session.globals.size(); i++)
-    {
-      vardecl* v = session.globals[i];
-      string name = v->unmangled_name;
-      if (s->code.find("/* pragma:read:" + name + " */") != string::npos)
-        {
-          if (v->type == pe_stats)
-            throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), s->tok);
-          read.insert(v);
-        }
-      if (s->code.find("/* pragma:write:" + name + " */") != string::npos)
-        {
-          if (v->type == pe_stats)
-            throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), s->tok);
-          written.insert(v);
-        }
-    }
+  // collect vardecls earlier identified during sym resolution
+  read.insert(s->read_referents.begin(), s->read_referents.end());
+  written.insert(s->write_referents.begin(), s->write_referents.end());  
 
-  // Don't allow embedded C functions in unprivileged mode unless
-  // they are tagged with /* unprivileged */ or /* myproc-unprivileged */
-  // or we're in a usermode runtime.
-  if (! pr_contains (session.privilege, pr_stapdev) &&
-      ! pr_contains (session.privilege, pr_stapsys) &&
-      ! session.runtime_usermode_p () &&
-      s->code.find ("/* unprivileged */") == string::npos &&
-      s->code.find ("/* myproc-unprivileged */") == string::npos)
-    throw SEMANTIC_ERROR (_F("function may not be used when --privilege=%s is specified",
-			     pr_name (session.privilege)),
-			  current_function->tok);
+  // check types
+  for (auto it = s->read_referents.begin(); it != s->read_referents.end(); it++) {
+    vardecl* v = *it;
+    if (v->type == pe_stats)
+      throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), v->tok);
+  }
+  for (auto it = s->write_referents.begin(); it != s->write_referents.end(); it++) {
+    vardecl* v = *it;
+    if (v->type == pe_stats)
+      throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), v->tok);
+  }
 
-  // Allow only /* bpf */ functions in bpf mode.
-  if ((session.runtime_mode == systemtap_session::bpf_runtime)
-      != (s->code.find ("/* bpf */") != string::npos))
-    {
-      if (session.runtime_mode == systemtap_session::bpf_runtime)
-	throw SEMANTIC_ERROR (_("function may not be used with bpf runtime"),
-                              current_function->tok);
-      else
-	throw SEMANTIC_ERROR (_("function requires bpf runtime"),
-                              current_function->tok);
-    }
-
-  // Don't allow /* guru */ functions unless -g is active.
-  if (!session.guru_mode && s->code.find ("/* guru */") != string::npos)
-    throw SEMANTIC_ERROR (_("function may not be used unless -g is specified"),
-			  current_function->tok);
+  
+  // NB: we used to do security/safety checks here, which are now over
+  // in elaborate.cxx functioncall_security_check because they may be
+  // call-site-sensitive.
 
   // PR14524: Support old-style THIS->local syntax on per-function basis.
-  if (s->code.find ("/* unmangled */") != string::npos)
+  if (s->tagged_p ("/* unmangled */"))
     current_function->mangle_oldstyle = true;
 
   // We want to elide embedded-C functions when possible.  For
@@ -2592,7 +2714,7 @@ varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
   // $target variables as rvalues will have this; lvalues won't.
   // Also, explicit side-effect-free tapset functions will have this.
 
-  if (s->code.find ("/* pure */") != string::npos)
+  if (s->tagged_p ("/* pure */"))
     return;
 
   embedded_seen = true;
@@ -2603,24 +2725,21 @@ varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
 void
 varuse_collecting_visitor::visit_embedded_expr (embedded_expr *e)
 {
-  /* We need to lock globals that are accessed through embedded C code */
-  for (unsigned i = 0; i < session.globals.size(); i++)
-    {
-      vardecl* v = session.globals[i];
-      string name = v->unmangled_name;
-      if (e->code.find("/* pragma:read:" + name + " */") != string::npos)
-        {
-          if (v->type == pe_stats)
-            throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), e->tok);
-          read.insert(v);
-        }
-      if (e->code.find("/* pragma:write:" + name + " */") != string::npos)
-        {
-          if (v->type == pe_stats)
-            throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), e->tok);
-          written.insert(v);
-        }
-    }
+  // collect vardecls earlier identified during sym resolution
+  read.insert(e->read_referents.begin(), e->read_referents.end());
+  written.insert(e->write_referents.begin(), e->write_referents.end());  
+
+  // check types
+  for (auto it = e->read_referents.begin(); it != e->read_referents.end(); it++) {
+    vardecl* v = *it;
+    if (v->type == pe_stats)
+      throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), v->tok);
+  }
+  for (auto it = e->write_referents.begin(); it != e->write_referents.end(); it++) {
+    vardecl* v = *it;
+    if (v->type == pe_stats)
+      throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), v->tok);
+  }
 
   // Don't allow embedded C expressions in unprivileged mode unless
   // they are tagged with /* unprivileged */ or /* myproc-unprivileged */
@@ -2628,14 +2747,14 @@ varuse_collecting_visitor::visit_embedded_expr (embedded_expr *e)
   if (! pr_contains (session.privilege, pr_stapdev) &&
       ! pr_contains (session.privilege, pr_stapsys) &&
       ! session.runtime_usermode_p () &&
-      e->code.find ("/* unprivileged */") == string::npos &&
-      e->code.find ("/* myproc-unprivileged */") == string::npos)
+      ! e->tagged_p ("/* unprivileged */") &&
+      ! e->tagged_p ("/* myproc-unprivileged */"))
     throw SEMANTIC_ERROR (_F("embedded expression may not be used when --privilege=%s is specified",
 			     pr_name (session.privilege)),
 			  e->tok);
 
   // Don't allow /* guru */ functions unless -g is active.
-  if (!session.guru_mode && e->code.find ("/* guru */") != string::npos)
+  if (!session.guru_mode && e->tagged_p ("/* guru */"))
     throw SEMANTIC_ERROR (_("embedded expression may not be used unless -g is specified"),
 			  e->tok);
 
@@ -2649,7 +2768,7 @@ varuse_collecting_visitor::visit_embedded_expr (embedded_expr *e)
   // $target variables as rvalues will have this; lvalues won't.
   // Also, explicit side-effect-free tapset functions will have this.
 
-  if (e->code.find ("/* pure */") != string::npos)
+  if (e->tagged_p ("/* pure */"))
     return;
 
   embedded_seen = true;
@@ -3246,6 +3365,12 @@ throwing_visitor::visit_defined_op (defined_op* e)
   throwone (e->tok);
 }
 
+void
+throwing_visitor::visit_probewrite_op(probewrite_op* e)
+{
+  throwone(e->tok);
+}
+
 void
 throwing_visitor::visit_entry_op (entry_op* e)
 {
@@ -3576,6 +3701,12 @@ update_visitor::visit_defined_op (defined_op* e)
   provide (e);
 }
 
+void
+update_visitor::visit_probewrite_op(probewrite_op* e)
+{
+  provide(e);
+}
+
 void
 update_visitor::visit_entry_op (entry_op* e)
 {
@@ -3891,7 +4022,16 @@ void
 deep_copy_visitor::visit_functioncall (functioncall* e)
 {
   functioncall* n = new functioncall(*e);
-  n->referents.clear(); // don't copy!
+  
+  // PR25841 ... we used to clear n->referents(), anticipating a later
+  // symbol resolution pass to replace all the functiondecl*'s, for
+  // reasons lost to history.  But in the face of function cloning,
+  // and the peculiar naming conventions there, ::find_functions()
+  // can't easily do the right thing.  So let's just preserve the
+  // referents.
+  //
+  // n->referents.clear();
+
   update_visitor::visit_functioncall(n);
 }
 
