@@ -99,6 +99,10 @@ public:
   void emit_module_decls (systemtap_session& s);
   void emit_module_init (systemtap_session& s);
   void emit_module_exit (systemtap_session& s);
+
+  friend void warn_for_bpf(systemtap_session& s,
+                           utrace_derived_probe_group *upg,
+                           const std::string& kind);
 };
 
 
@@ -1300,6 +1304,42 @@ utrace_derived_probe_group::emit_module_exit (systemtap_session& s)
 }
 
 
+// PR26234: Not supported by stapbpf.
+void
+warn_for_bpf(systemtap_session& s, utrace_derived_probe_group *upg,
+             const std::string& kind)
+{
+  if (! upg->probes_by_path.empty())
+    {
+      for (utrace_derived_probe_group::p_b_path_iterator it
+             = upg->probes_by_path.begin();
+           it != upg->probes_by_path.end(); it++)
+        {
+          for (unsigned i = 0; i < it->second.size(); i++)
+            {
+              s.print_warning(_F("%s will be ignored by bpf backend",
+                                 kind.c_str()),
+                              it->second[i]->tok);
+            }
+        }
+    }
+  if (! upg->probes_by_pid.empty())
+    {
+      for (utrace_derived_probe_group::p_b_pid_iterator it
+             = upg->probes_by_pid.begin();
+           it != upg->probes_by_pid.end(); it++)
+        {
+          for (unsigned i = 0; i < it->second.size(); i++)
+            {
+              s.print_warning(_F("%s will be ignored by bpf backend",
+                                 kind.c_str()),
+                              it->second[i]->tok);
+            }
+        }
+    }
+}
+
+
 void
 register_tapset_utrace(systemtap_session& s)
 {
