@@ -7,5 +7,5 @@ func_count: 1}
 
 # Only run on make installcheck
 if {! [installtest_p]} { untested "lib-$testname"; return }
-if {! [utrace_p]} { untested "lib-$testname"; return }
+if {! [uprobes_p]} { untested "lib-$testname"; return }
 stap_run3 mark-$testname $srcdir/$subdir/mark.stp $testexe $testlib -c $testexe
